#!/bin/bash
# $Id: configure.sh,v 1.17 2007-05-19 17:08:03 devel Exp $
# Author   : Jim Brooks http://www.jimbrooks.org
# Synopsis : Simple configuration script for pypersrc on GNU systems.
# Notes    :
# --------------------------------------------------------------------------

TOOLS_DIR=tools
MAKEFILE=Makefile

# [2009/09]
# pypersrc depends on Pmw which requires Python 2.x.
if ( ! python tools/check_python2.py ); then
	# Print additional info.
	echo "pypersrc depends on Pmw which hasn't been ported to Python 3.x yet [2009/09]."
    exit 1
fi

# Depending on system, determine name of GNU make and create a makefile.
if [ "`uname -s`" = "Darwin" ]; then
    GMAKE=make
elif ( uname -a | grep -q BSD ); then
	# BSD.
	GMAKE=gmake
	# Create a traditional makefile which defers to GNU make.
	echo "#--- AUTO-GENERATED by configure.sh on `uname` ---"  > $MAKEFILE
	echo "all .DEFAULT:"                                      >> $MAKEFILE
	# This echo starts with a TAB (echo -e doesn't work on Solaris).
	echo "	${GMAKE} -f GNUmakefile \$@"                      >> $MAKEFILE
else
	# GNU/Linux.
	GMAKE=make
fi

# Ensure GNU make is installed.
if [ "`$GMAKE --version 2>/dev/null | grep GNU`" = "" ]; then
	echo "ERROR: GNU make is required."
	exit 1
fi

# Check for and configure Python.
python -h >/dev/null 2>/dev/null
if [ "$?" = 127 ]; then
    echo "ERROR: python is required"
    exit 1
fi
MIN_PYTHON_VERSION=2.3
PYTHON_CC_FLAGS=`python ${TOOLS_DIR}/pythonconf.py -min $MIN_PYTHON_VERSION -cc-flags`
PYTHON_INC=`python ${TOOLS_DIR}/pythonconf.py -min $MIN_PYTHON_VERSION -inc`
PYTHON_LD_FLAGS=`python ${TOOLS_DIR}/pythonconf.py -min $MIN_PYTHON_VERSION -ld-flags`
PYTHON_LD_LIB_FLAGS=`python ${TOOLS_DIR}/pythonconf.py -min $MIN_PYTHON_VERSION -ld-lib-flags`
PYTHON_LIB=`python ${TOOLS_DIR}/pythonconf.py -min $MIN_PYTHON_VERSION -lib`
PYTHON_INSTALL_DIR=`python ${TOOLS_DIR}/pythonconf.py -min $MIN_PYTHON_VERSION -install-dir`
if [ "$PYTHON_INC" = "" ]; then
    echo "ERROR: failure running python script"
    exit 1
fi

# Check for Tkinter.
if ( ! python ${TOOLS_DIR}/check.py -tkinter ); then
    echo "ERROR: Python Tkinter module is required."
    echo "On GNU systems: Tkinter may be an optional Python package."
    echo "On Mac OS X: Fink's version of Python supplies Tkinter."
    exit 1
fi

# Check for Python Mega Widgets (Pmw).
if ( ! python ${TOOLS_DIR}/check.py -pmw ); then
    echo "ERROR: Python Mega Widgets (Pmw) is required. "
	echo "Download it from http://pmw.sourceforge.net/"
	echo "Pmw is easy to install: just untar it into Python's site-packages/ subdir."
	echo "/usr/lib/python*/site-packages is typical on UNIX systems"
    exit 1
fi

# Where to install executables/scripts.
if [ "$USER" = "root" ]; then
    CC_INSTALL_DIR=/usr/local/bin
else
    CC_INSTALL_DIR=~/bin
fi

# Create config file to be included by GNUmakefile.
echo "#--- AUTO-GENERATED by configure.sh on `uname` ---"      > GNUmakefile.config
echo "CC_FLAGS           += ${PYTHON_CC_FLAGS} ${PYTHON_INC}" >> GNUmakefile.config
echo "LD_FLAGS           += ${PYTHON_LD_FLAGS}"               >> GNUmakefile.config
echo "LD_LIB_FLAGS       += ${PYTHON_LD_LIB_FLAGS}"           >> GNUmakefile.config
echo "LIBS               += ${PYTHON_LIB}"                    >> GNUmakefile.config
# Quotes in case of $HOME on cygwin.
echo "CC_INSTALL_DIR      = \"${CC_INSTALL_DIR}\""            >> GNUmakefile.config
echo "PYTHON_INSTALL_DIR  = \"${PYTHON_INSTALL_DIR}\""        >> GNUmakefile.config

# Done.
echo "Successfully configured on `uname`.  Ready for ${GMAKE}."

exit 0
