/*****************************************************************************
 * $Id: utils_file.cc,v 1.4 2006-02-28 14:24:37 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Utility file functions.
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) JIM E. BROOKS 2005
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <iostream>
#include <string>
using namespace std;

#include "utils_file.hh"

namespace UtilsNS {

/*****************************************************************************
 * Determine full pathname of the currently running executable.
 * The returned path will have a trailing slash '/'.
 *****************************************************************************/
bool
GetExecPath( const char* pArgv0, string& execPath/*OUT*/ )
{
	execPath = "";

	// If argv[0] has no slashes, then PATH is referenced,
	// so PATH must be parsed.  Otherwise, argv[0] is either
	// a full pathname or a pathname relative to CWD.
	if ( strchr( pArgv0, '/' ) != NULL )
	{
		// Found a slash.
		if ( pArgv0[0] == '/' )
		{
			// argv[0] holds absolute path.
			execPath.assign( pArgv0, static_cast<const char*>(strrchr(pArgv0,'/')+1) );
			return true;
		}
		else
		{
			// argv[0] holds relative path.
			char cwd[256];
			if ( getcwd( cwd, sizeof(cwd) ) == NULL )
                return false;
			execPath = cwd;
			execPath += '/';
			const char* lastSlash = strrchr( pArgv0, '/' );
			if ( lastSlash == NULL )
			{
				return true;
			}
			else
			{
				string path2;
				path2.assign( pArgv0, lastSlash+1 );
				execPath += path2;
				return true;
			}
		}
	}
	else
	{
		// No slash found.  Parse PATH.
		const char* pc1;
		const char* pc2;
		const char* path = getenv( "PATH" );
		if ( path == NULL ) return false;
		int pathLen = strlen( path );
		string file;
		for ( pc1 = path; pc1 < &path[pathLen]; pc1 = pc2 + 1 )
		{
			pc2 = strchr( pc1, ':' );
			if ( pc2 == NULL )
				pc2 = &path[pathLen];
			execPath.assign( pc1, pc2 );
			execPath += '/';
			file = execPath;
			file += pArgv0;
			if ( access( file.c_str(), R_OK|X_OK ) == 0 )
				return true;
		}
		execPath = "";
		return false;
	}
	return false;
}

} // namespace UtilsNS
