/*****************************************************************************
 * $Id: utils_funcs.hh,v 1.2 2006-02-28 14:24:37 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Utility misc functions.
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) JIM E. BROOKS 2006
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef UTILS_FUNCS_HH
#define UTILS_FUNCS_HH 1
namespace UtilsNS {

// -----------------------------------------------------------------------------
// C++ STL helpers, extensions.

/*****************************************************************************
 * Find a subset in two containers of different types but whose elements
 * are of the same type.  This is an alternative to STL search() which
 * is limited to containers of the same types.
 * @return If found, returns an iterator in container #1, else returns it1End.
 *****************************************************************************/
template<class T1, class T2>
typename T1::iterator
FindSubset( typename T1::iterator it1Beg, typename T1::iterator it1End,
            typename T2::iterator it2Beg, typename T2::iterator it2End )
{
	typename T1::iterator it1;  // compiler forbids declaring inside of for()
	typename T2::iterator it2;
	for ( it1 = it1Beg; it1 != it1End; ++it1 )
	{
		typename T1::iterator it1b = it1;
		for ( it2 = it2Beg; it2 != it2End; ++it2, ++it1b )
		{
			if ( it1b == it1End ) return it1End;	// reached end of container #1
			if ( *it1b != *it2 ) break;				// char mismatch
		}
		if ( it2 == it2End ) return it1; // found match
	}
	return it1End; // no match
}

} // namespace UtilsNS
#endif // UTILS_FUNCS_HH
