/*****************************************************************************
 * $Id: utils_macros.hh,v 1.2 2006-02-28 14:24:37 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Utility macros.
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) JIM E. BROOKS 2006
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef UTILS_MACROS_HH
#define UTILS_MACROS_HH 1
namespace UtilsNS {

#ifndef STREQ
#define STREQ( S1, S2 )			( strcmp( (S1), (S2) ) == 0 )
#endif

#ifndef STRNE
#define STRNE( S1, S2 )			( strcmp( (S1), (S2) ) != 0 )
#endif

#ifndef STREMPTY
#define STREMPTY( S )           ( ((S) == NULL) || ((S)[0] == '\0') )
#endif

#ifndef MEMEQ
#define MEMEQ( S1, S2, LEN )	( memcmp( (S1), (S2), (LEN) ) == 0 )
#endif

#ifndef MENNE
#define MEMNE( S1, S2, LEN )	( memcmp( (S1), (S2), (LEN) ) != 0 )
#endif

// Use macro MIN()/MAX() if both args are data values.
// Use template Min()/Max() if either arg is a computation, function call,
// or data referenced by a pointer.
//
// MIN()/MAX(): Simple but inefficient if A or B is a computation.
#ifndef MIN
#define MIN(A,B) ( ((A) < (B)) ? (A) : (B) )
#endif
#ifndef MAX
#define MAX(A,B) ( ((A) > (B)) ? (A) : (B) )
#endif
// Min()/Max(): Faster if A or B is a computation.
template<class T> inline T Min( T a, T b ) { return (a < b) ? a : b; }
template<class T> inline T Max( T a, T b ) { return (a > b) ? a : b; }

#define ELEMS( ARRAY )	        ( sizeof(ARRAY) / sizeof(ARRAY[0]) )

} // namespace UtilsNS
#endif // UTILS_MACROS_HH
