/*****************************************************************************
 * $Id: utils_string.cc,v 1.2 2006-02-28 14:24:37 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Utility string functions.
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) JIM E. BROOKS 2006
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include <iostream>
#include <string>
using namespace std;

#include "utils_string.hh"

namespace UtilsNS {

/******************************************************************************
 * Find a substring, ignoring case.
 * @pre		haystack,needle MUST NOT BE NULL !!
 *			CALLER MUST INITIALIZE line to 1 !!
 * @return	Pointer to needle in haystack or NULL if no match.
 *			line is UNDEFINED if NULL is returned.
 * needle="" is treated as no match (this behavior differs from strcasestr()).
 * Why isn't strcasestr() used?  Because it's not a part of stdlib.
 ******************************************************************************/
const char*
Find( const char* haystack, const char* needle, bool caseSens, int& line )
{
	// For each char in haystack.
	const char* haystack1 = haystack;
    for ( ; *haystack1; ++haystack1 )
	{
        if ( *haystack1 == '\n' ) ++line;

		// Test if needle matches this spot in haystack.
		const char* haystack2 = haystack1;
		const char* needle2 = needle;
		for ( ; ; ++haystack2,++needle2 )
		{
			int h, n;
			if ( caseSens )    // CPU branch prediction will amortize this
			{
				h = *haystack2;
				n = *needle2;
			}
			else
			{
				// Convert chars to lower-case for case-insensitivity.
				h = tolower( *haystack2 );
				n = tolower( *needle2 );
			}

			// Hit end of needle?
			if ( n == 0 )
			{
				// needle="" reaches this point, treat it as no match.
				if ( *needle )
					return haystack1;
				else
					return NULL;
			}
			// Hit end of haystack?
			else if ( h == 0 )
			{
				return NULL;
			}
			// Chars mismatch?
			else if ( h != n )
			{
				break;
			}
			// Chars match.  Needle matches partially so far.  Continue.
		}

		// Needle didn't match this spot in haystack.
	}

	// Needle wasn't found.
	return NULL;
}

} // namespace UtilsNS
