# $Id: DequeChar.py,v 1.6 2006-02-25 03:56:48 devel Exp $
# Python DequeChar class built over underlying C++ STL deque<char>.
# Author:   Jim Brooks  http://www.jimbrooks.org
# License:  GNU General Public License (GPL)
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

#-------------------------------------------------------------------------------
# Class DequeChar.
#-------------------------------------------------------------------------------

import pycpp_dequechar

class DequeChar(object):
    """ DequeChar is based on C++ STL deque<char> """

    def __init__( s, chars="" ):
        pycpp_dequechar.DequeChar_New( s )
        pycpp_dequechar.DequeChar_Set( s, chars )

    def __del__( s ):
        # An exception will normally occur during program exit
        # because modules including pycpp_dequechar are gone.
        try:
            pycpp_dequechar.DequeChar_Delete( s )
        except:
            pass

    def PopHead( s, n=1 ):
        """ Remove n chars as a string from head.
            Raises exception if not all n chars are available. """
        return pycpp_dequechar.DequeChar_PopHead( s, n )

    def PopTail( s, n=1 ):
        """ Remove n chars as a string from tail.
            Raises exception if not all n chars are available. """
        return pycpp_dequechar.DequeChar_PopTail( s, n )
        
    def PeekHead( s, n=1 ):
        """ Peek n chars as a string at head without removing them.
            Raises exception if not all n chars are available. """
        return pycpp_dequechar.DequeChar_PeekHead( s, n )

    def PeekTail( s, n=1 ):
        """ Peek n chars as a string at tail without removing them.
            Raises exception if not all n chars are available. """
        return pycpp_dequechar.DequeChar_PeekTail( s, n )

    def PushHead( s, chars ):
        """ Enqueue a string at head. """
        pycpp_dequechar.DequeChar_PushHead( s, chars )

    def PushTail( s, chars ):
        """ Enqueue a string at tail. """
        pycpp_dequechar.DequeChar_PushTail( s, chars )

    def IfEmpty( s ):
        """ True if deque is empty. """
        return pycpp_dequechar.DequeChar_IfEmpty( s )

    def Size( s ):
        """ Return amount of elements in deque. """
        return pycpp_dequechar.DequeChar_Size( s )

    def Set( s, ibuf ):
        """ Set the contents of deque from a string.
            To clear the deque, pass an empty string. """
        pycpp_dequechar.DequeChar_Set( s, ibuf )

    def Get( s ):
        """ Return contents of deque as a string. """
        return pycpp_dequechar.DequeChar_Get( s )

    def __setitem__( s, idx, c ):
        """ Assign to a char by index in deque. """
        # A negative index within range is allowed.
        pycpp_dequechar.DequeChar_SetItem( s, idx, c )

    def __getitem__( s, idx ):
        """ Return a char by index from deque. """
        # A negative index within range is allowed.
        return pycpp_dequechar.DequeChar_GetItem( s, idx )

    def Index( s, sought, idxStart=0 ):
        """ Return index of substring in deque starting at an index. """
        # Raises exception if no match, sought string is empty,
        # or index plus sought string is past end of deque.
        return pycpp_dequechar.DequeChar_Index( s, sought, idxStart )
