# $Id: GreedyDict.py,v 1.5 2006-02-25 03:56:48 devel Exp $
# Python GreedyDict built over underlying C++ GreedMap template class.
# Author:   Jim Brooks  http://www.jimbrooks.org
# License:  GNU General Public License (GPL)
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

#-------------------------------------------------------------------------------
# Class GreedyDict.
#-------------------------------------------------------------------------------

import pycpp_greedydict

class GreedyDict(object):
    """ Dictionary that greedily finds the longest matching key. """

    def __init__( s, hintKeySize=15 ):
        pycpp_greedydict.GreedyDict_New( s, hintKeySize )
        return

    # INTERFACE
    def __del__( s ):
        # An exception will normally occur during program exit
        # because modules including pycpp_greeydict are gone.
        try:
            pycpp_greedydict.GreedyDict_Delete( s )
        except:
            pass
        return

    # INTERFACE
    def __setitem__( s, key, val ):
        """ Insert a key,value into the dictionary. """
        pycpp_greedydict.GreedyDict_Insert( s, key, val )

    # INTERFACE
    def __getitem__( s, key ):
        """ Return a value from the dictionary. """
        # This match do an approximate match (see GreedyMap.hh).
        (key2,val) = pycpp_greedydict.GreedyDict_Find( s, key )
        return val

    # INTERFACE
    def Find( s, key ):
        """ Return (key,value) from the dictionary.
            The returned key is the actual matching key which
            may or may not exactly be the same as the passed key. """
        return pycpp_greedydict.GreedyDict_Find( s, key )
