# $Id: GuiTk.py,v 1.50 2006-03-26 03:03:32 devel Exp $
# pypersrc
# Author:   Jim Brooks  http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# Notes:    On cygwin, event.keysym isn't assigned correctly for Fn keys,
#           so they're bound to individual handlers.
# ==============================================================================

#-------------------------------------------------------------------------------
# Load-time checks.
#-------------------------------------------------------------------------------

# Even if configure.sh approved, the environment or Python installation could change.

if IfCygwin() and not os.environ.has_key("TCL_LIBRARY"):
    sys.stderr.write( "###\n" )
    sys.stderr.write( "### TCL_LIBRARY env var must be set, for example: \n" )
    sys.stderr.write( "### export TCL_LIBRARY=c:/cygwin/usr/share/tcl8.4 \n" )
    sys.stderr.write( "###\n" )

# Although Tkinter is supposedly a standard part of Python, in practice,
# not all systems have Tk installed (nor Tkinter consequently).
# This is necessary to print a meaningful error for the user.
try:
    import Tkinter
except:
    sys.stderr.write( "### pypersrc depends on Tk/Tkinter which doesn't seem to be installed. \n" )
    sys.stderr.write( "### Or, less likely, the pypersrc executable can't be loaded because \n" )
    sys.stderr.write( "### of a shared lib dependency. \n" )
    if IfDarwin():
        sys.stderr.write( "### If this is Mac OS X older than 10.4 Tiger, try installing \n" )
        sys.stderr.write( "### Fink's version of Python. \n" )
        sys.stderr.write( "### http://fink.sourceforge.net \n" )
TK = Tkinter    # abbrev
import tkFont

# Pmw is an required site-specific module based on Tkinter.
try:
    import Pmw
except:
    sys.stderr.write( "### pypersrc depends on Python Mega Widgets (Pmw). \n" )
    sys.stderr.write( "### Download: http://pmw.sourceforge.net/ \n" )
    sys.stderr.write( "### Pmw needs to be installed in Python's site-packages/ subdir. \n" )
    sys.stderr.write( "### If your system has multiple installations of Python, \n" )
    sys.stderr.write( "### install Pmw at which one pypersrc is linked with. \n" )
    if IfDarwin():
        sys.stderr.write( "### On Mac OS X, pypersrc is linked with Apple's version of Python \n" )
        sys.stderr.write( "### which is at /usr.  Fink's Pmw package, if installed, is at /sw \n" )
        sys.stderr.write( "### ln -s /sw/lib/python2.3/site-packages/Pmw /usr/lib/python2.3/site-packages/Pmw \n" )

#-------------------------------------------------------------------------------
# Class Gui.
#-------------------------------------------------------------------------------

class Gui(GuiBase):
    """ pypersrc Tkinter GUI. """

    def __init__( s, argList ):
        GuiBase.__init__( s, argList )

        s.ready           = False
        s.beenEnabled     = False
        s.keyEventTime    = 0.0         # to mitigate fast keyboard repeat rate
        
        s.TKTAG_DEFAULT   = "df"
        s.TKTAG_TEXT      = "tx"
        s.TKTAG_NUM       = "nu"
        s.TKTAG_SYMBOL    = "sy"
        s.TKTAG_WORD      = "wo"
        s.TKTAG_KEYWORD   = "kw"
        s.TKTAG_TYPE      = "ty"
        s.TKTAG_LOUD      = "lo"
        s.TKTAG_QUOTED    = "qu"
        s.TKTAG_COMMENT   = "co"
        s.TKTAG_TAGDEF    = "td"
        s.TKTAG_TAGREF    = "tr"
        s.TKTAG_TAGREFV   = "trv"  # visited hyperlink
        
        # Lexical tokens are drawn with Tk text widget tags
        # (not to be confused with source code tags).
        s.tktagDict = { eTokenInvalid     : s.TKTAG_DEFAULT,
                        eTokenWs          : s.TKTAG_DEFAULT,
                        eTokenSymbol      : s.TKTAG_SYMBOL,
                        eTokenNum         : s.TKTAG_NUM,
                        eTokenWord        : s.TKTAG_WORD,
                        eTokenKeyword     : s.TKTAG_KEYWORD,
                        eTokenKeywordLoud : s.TKTAG_LOUD,
                        eTokenType        : s.TKTAG_TYPE,
                        eTokenText        : s.TKTAG_TEXT,
                        eTokenTag         : s.TKTAG_TAGDEF,
                        #eTokenTag        : s.TKTAG_TAGREF,  # special case
                        #eTokenTag        : s.TKTAG_TAGREFV, # special case
                        eTokenQuoted      : s.TKTAG_QUOTED,
                        eTokenComment     : s.TKTAG_COMMENT,
                        }
        
        # Set main window geometry.
        s.root = TK.Tk()
        Pmw.initialise( s.root )
        s.root.geometry( s.geom )

        # These handlers must be installed right now.
        # Other handlers are installed by InstallHandlers() when GUI becomes enabled.
        # -- TextWid.__HandlerKey() handles keyboard events in text widgets first --
        s.root.bind( "<KeyPress-Escape>",   s.__HandlerKeyPress )
        s.root.bind( "<KeyRelease-Escape>", s.__HandlerKeyRelease )
        s.root.bind( "<Destroy>", s.__HandlerProgramExit ) # this intercepts ALL destroy events!

        # Create the main frame.
        s.mainFrame = TK.Frame( s.root )
        s.mainFrame.pack( expand=TK.YES, fill=TK.BOTH )

        s.SetTitle( "pypersrc", verbatim=True )

        # Create the statusbar.
        s.statusbar = Statusbar( s.mainFrame )

        # Create Command entry and Filter checkbuttons at top in a frame.
        s.entryFilterFrame = TK.Frame( s.mainFrame, relief=TK.RAISED, borderwidth=2 )
        s.entryFilterFrame.pack( side=TK.TOP, expand=TK.NO, fill=TK.BOTH, anchor=TK.W )

        s.entryFrame = TK.Frame( s.entryFilterFrame )
        s.entryFrame.pack( side=TK.LEFT, expand=TK.YES, fill=TK.BOTH, anchor=TK.W )

        s.filterFrame = TK.Frame( s.entryFilterFrame )
        s.filterFrame.pack( side=TK.RIGHT, anchor=TK.E )

        s.labelCommand = TK.Label( s.entryFrame )
        s.labelCommand.config( text="Command: " )
        s.labelCommand.pack( side=TK.LEFT, anchor=TK.W )
        
        s.colorFocusOut = s.labelCommand["bg"]
        s.colorFocusIn  = "#a0a0f0"

        s.entryCommand = TK.Entry( s.entryFrame )
        s.entryCommand.pack( side=TK.LEFT, anchor=TK.W )
        st.gui.DisableWidget( s.entryCommand )

        s.labelFilter = TK.Label( s.filterFrame )
        s.labelFilter.config( text="Filter: " )
        s.labelFilter.pack( side=TK.LEFT, anchor=TK.E )

        s.__ResetFilterButtons()
        
        s.checkButtonComments  = TK.Checkbutton( s.filterFrame, text="comments",  command=s.__HandlerFilterComments )
        s.checkButtonFunctions = TK.Checkbutton( s.filterFrame, text="functions", command=s.__HandlerFilterFunctions )
        s.checkButtonData      = TK.Checkbutton( s.filterFrame, text="data",      command=s.__HandlerFilterData )
        
        st.gui.DisableWidget( s.checkButtonComments )
        st.gui.DisableWidget( s.checkButtonFunctions )
        st.gui.DisableWidget( s.checkButtonData )
        
        s.checkButtonComments.pack(  side=TK.LEFT, anchor=TK.E )
        s.checkButtonFunctions.pack( side=TK.LEFT, anchor=TK.E )
        s.checkButtonData.pack( side=TK.LEFT, anchor=TK.E )

        s.buttonRefresh = TK.Button( s.filterFrame, text="refresh", command=s.__HandlerButtonRefresh )
        s.buttonRefresh.pack( side=TK.LEFT, anchor=TK.E )
        st.gui.DisableWidget( s.buttonRefresh )

        # Yes, the Close (Page) button is separate from the notebook widget,
        # as here is the right time to pack the button.
        # Consequently, the Gui class, rather than the Notebook class, handles this button
        s.buttonClosePage = TK.Button( s.filterFrame, text="close", command=s.__HandlerButtonClosePage )
        s.buttonClosePage.pack( side=TK.LEFT, anchor=TK.E )
        st.gui.DisableWidget( s.buttonClosePage )

        # Create horizontal paned widget for listbox and vertical paned widget.
        panedArgs = { "separatorthickness":4, "separatorrelief":"raised", "handlesize":12 }
        s.panedHorz = Pmw.PanedWidget( s.mainFrame, orient="horizontal", **panedArgs )
        s.panedHorz.pack( expand=1, fill="both" )
        s.panedHorz1 = s.panedHorz.add( "listbox",   size=0.20 )
        s.panedHorz2 = s.panedHorz.add( "panedVert", size=0.80 )
        s.panedHorz1.pack()
        s.panedHorz2.pack()

        # Create vertical paned widget for both text widgets.
        s.panedVert = Pmw.PanedWidget( s.panedHorz2, orient="vertical", **panedArgs )
        s.panedVert.pack( expand=1, fill="both" )
        s.panedVert1 = s.panedVert.add( "text1", size=0.75 )
        s.panedVert2 = s.panedVert.add( "text2", size=0.25 )
        s.panedVert1.pack()
        s.panedVert2.pack()

        # Create source file listbox.
        s.fileWid = ScrollFrame( s.panedHorz1, TK.Listbox, "", selectmode=TK.SINGLE )
        s.fileWid.frame.pack( side=TK.BOTTOM, expand=TK.YES, fill=TK.BOTH )

        # Create notebook to hold text widgets (tabbed browsing of src code).
        s.notebookSrcFile = NotebookSrcFile( s.panedVert1 )
        
        # Create a text widget to show results.
        # -- clients should access this via GetResultsTextWid() --
        s.resultsTextWid = TextWidResults( s.panedVert2 )

        # Create a cache to speed reloading text widgets.
        s.cacheSrcFile = CacheSrcFile()

        # Create history of results.
        s.resultsHistory = ResultsHistory()

        s.ProcessArgsPost( argList )

        # Let's rock.
        s.ready = True
        
    #-------------------------------------------------------------------------------
    # Interface methods.
    #-------------------------------------------------------------------------------

    # INTERFACE
    def IfReady( s ):
        return s.ready

    # INTERFACE
    def Enable( s, enable=True ):
        """ Make GUI sensitive to input. """

        # Allow nesting Gui.Enable(True/False).
        # Only the outer level has an effect.
        if not s.nestEnable.IfProceed( enable ):
            return

        # Proceed (now at outer-level of nesting).
        GuiBase.Enable( s, enable )  # sets s.enabled

        # Install handlers only once.
        if enable and not s.beenEnabled:
            s.beenEnabled = True
            s.__InstallHandlers()

        # Enable all the widgets.
        st.gui.EnableWidget( s.labelCommand, enable )
        st.gui.EnableWidget( s.entryCommand, enable )
        st.gui.EnableWidget( s.labelFilter, enable )
        st.gui.EnableWidget( s.checkButtonComments, enable )
        st.gui.EnableWidget( s.checkButtonFunctions, enable )
        st.gui.EnableWidget( s.checkButtonData, enable )
        st.gui.EnableWidget( s.buttonRefresh, enable )
        s.fileWid.Enable( enable )
        s.resultsTextWid.Enable( enable )
        s.notebookSrcFile.Enable( enable )
        if not st.gui.notebookSrcFile.IfEmpty():
            st.gui.EnableWidget( s.buttonClosePage, enable ) # Gui class manages this button
        else:
            st.gui.DisableWidget( s.buttonClosePage ) # should be disabled if no pages
        #if enable:
        #    s.entryCommand.focus()

        # To take effect.
        st.gui.ProcessPendingEvents()

    # INTERFACE
    def EnableWidget( s, widget, enable=True ):
        """ Enable widget (or disable if enable=False). """
        # Should apply to Pmw widgets as they're based on Tk widgets.
        # FYI: This was supposed to return the old Tk widget state as bool,
        #      but determining that proved unreliable as some widgets have
        #      multiple enable states and some widgets don't implement querying "state".

        # Convert new enablement to a Tk "state" config arg.
        if enable:
            stateNew = TK.NORMAL
        else:
            stateNew = TK.DISABLED
        widget.config( state=stateNew )

    # INTERFACE
    def EventLoop( s ):
        s.root.mainloop()

    # INTERFACE
    def SetTitle( s, title, verbatim=False ):
        """ Set text of title bar of main window. """
        if verbatim:
            pre = ""
        else:
            pre = "pypersrc  "
        s.root.title( pre + title )

    # INTERFACE
    def SetFileList( s, fileList ):
        """ Set listbox with list of source filenames to browse. """

        listbox = s.fileWid.widget

        # Listbox must be enabled or it'll do nothing.
        enable = st.gui.__EnableWidgetIfNotGui( listbox, True )

        listbox.delete( "0", TK.END )              # clear listbox
        pos = 0
        for fname in fileList:                     # append filenames into listbox
            listbox.insert( pos, fname )
            pos += 1
        
        st.gui.__EnableWidgetIfNotGui( listbox, enable )  # restore enablement

    # INTERFACE
    def GetFileList( s ):
        """ Return list of source filenames. """
        return s.fileWid.widget.get( 0, TK.END )
    
    # INTERFACE
    def DeselectFileList( s, idx ):
        """ Deselect row(s) in the listbox of src files. """
        s.fileWid.widget.selection_clear( 0, TK.END )
    
    # INTERFACE
    def GetResultsTextWid( s ):
        """ Return the text widget used to hold results. """
        return s.resultsTextWid

    # INTERFACE
    def FlushSrcFiles( s, parseModes=None ):
        """ Flush out all source files from cache. """
        # Without args, flushes entire cache.
        # With parseModes=n, flushes src files filtered in specified parse modes.
        s.cacheSrcFile.Flush( parseModes=parseModes )

    # INTERFACE
    def ReloadSrcFiles( s ):
        """ Reload those text widgets that are showing source code. """
        s.notebookSrcFile.Reload()

    # INTERFACE
    def TagClick( s, clickedTagName, clickedFname, currentPage=False ):
        """ The user clicked a hyperlink in a text widget.
            Also called for the [click tag] internal command (for refreshing). """
        try:
            (goodTagList,badTagList) = st.tags.ChooseTag( clickedTagName, clickedFname )
            if not goodTagList and not badTagList:
                s.statusbar.Print( "Can't find the tag '" + clickedTagName + "'" )
            elif len(goodTagList) == 1 and not badTagList:
                # Only one tag has this name
                # Load the source file at the tagged line into the text widget.
                fname = goodTagList[0][TAG_FILE]
                line  = goodTagList[0][TAG_LINE]
                st.gui.ScheduleIdleFunc( s.notebookSrcFile.LoadSrcFile, fname, line, True, currentPage )
            else:
                # Multiple tags have the same name.

                # If a tag was clicked while filtering functions,
                # automatically choose the tag that is in the module being viewed.

                foundTheBestTag = False

                if st.parser.IfFiltering():
                    for tag in goodTagList:
                        if (st.gui.notebookSrcFile.GetCurrentTextWid()) \
                       and (st.gui.notebookSrcFile.GetCurrentTextWid().fname == tag[TAG_FILE]):
                            foundTheBestTag = True
                            fname = tag[TAG_FILE]
                            line  = tag[TAG_LINE]
                            st.gui.ScheduleIdleFunc( s.notebookSrcFile.LoadSrcFile, fname, line )
                        
                if not foundTheBestTag:
                    # Let the user pick the tag.
                    # Show a list hyperlinks to the tags in the results widget,
                    # and try to put the best matching hyperlink at the top.
                    tokenList = []
                    if goodTagList:
                        tokenList.append( (eTokenText,"Found tags with the same name, best match is first:\n\n") )
                    for tag in goodTagList:
                        text = tag[TAG_NAME] + " " + tag[TAG_TYPE] + " " + tag[TAG_FILE] + " " + str(tag[TAG_LINE])
                        phonyTag = st.tags.PhonyTag( text, tag[TAG_FILE], tag[TAG_LINE] )
                        tokenList.append( (eTokenTag,phonyTag) )
                    if badTagList:
                        if not goodTagList:
                            tokenList.append( (eTokenText,"Found tags with the same name, unlikely matches: \n\n") )
                        else:
                            tokenList.append( (eTokenText,"Unlikely matches: \n\n") )
                        for tag in badTagList:
                            text = tag[TAG_NAME] + " " + tag[TAG_TYPE] + " " + tag[TAG_FILE] + " " + str(tag[TAG_LINE])
                            phonyTag = st.tags.PhonyTag( text, tag[TAG_FILE], tag[TAG_LINE] )
                            tokenList.append( (eTokenTag,phonyTag) )
                    fname = "[results: click tag '" + clickedTagName + "' in " + clickedFname + "]"
                    # Compose internal command it needs to be run to refresh this later.
                    command = Command.INTERNAL_COMMAND_CLICK_TAG + " " + clickedTagName + " " + clickedFname
                    contents = ContentsResults( tokenList=tokenList, fname=fname, command=command )
                    s.GetResultsTextWid().LoadContents( contents )
        except:
            PrintException()

    # INTERFACE
    def ProcessPendingEvents( s ):
        """ Process any pending Tk events. """
        # Prevent Tk exception caused by a pending Quit event (program exit).
        if not st.exiting:
            s.root.update()
        if not st.exiting:
            s.root.update_idletasks()   # prevent Tk exception at this

    # INTERFACE
    def ScheduleIdleFunc( s, func, *args ):
        """ Schedule a function to be called (once) when GUI is idle. """
        s.root.after_idle( func, *args )

    # INTERFACE
    def ScheduleTimerFunc( s, milsec, func, *args ):
        """ Schedule a function to be called (once) in the next n milliseconds. """
        s.root.after( milsec, func, *args )

    #-------------------------------------------------------------------------------
    # Misc private methods.
    #-------------------------------------------------------------------------------

    # PRIVATE
    def __EnableWidgetIfNotGui( s, widget, enable=True ):
        """ If GUI is disabled, enable or disable widget.
            If GUI is enabled, NOP.
            Returns old enable state of widget. """
        # Tk-specific.  The purpose is to temporarily enable some widgets
        # while the entire GUI is supposed to be disabled, as otherwise
        # some widgets can't be changed when disabled.
        stateOld = widget.cget( "state" )
        if not s.IfEnabled():
            s.EnableWidget( widget, enable )  # enable or disable
        return stateOld

    # PRIVATE
    def __ResetFilterButtons( s ):
        s.checkButtonFlagComments  = False
        s.checkButtonFlagFunctions = False
        s.checkButtonFlagData      = False

    #-------------------------------------------------------------------------------
    # Event handlers.
    #-------------------------------------------------------------------------------

    # PRIVATE
    def __InstallHandlers( s ):
        s.beenEnabled = True
        # -- TextWid.__HandlerKey() handles keyboard events in text widgets first --
        s.root.bind( "<F1>",                 s.__HandlerKeyPress )
        s.root.bind( "<F2>",                 s.__HandlerKeyPress )
        s.root.bind( "<Control-w>",          s.__HandlerKeyPress )
       #s.fileWid.widget.bind( "<Button-1>", s.__HandlerFileListbox ) # doesn't work right
        s.fileWid.widget.bind( "<Double-1>", s.__HandlerFileListbox )
        s.entryCommand.bind( "<FocusIn>",    s.__HandlerFocusIn )
        s.entryCommand.bind( "<FocusOut>",   s.__HandlerFocusOut )
        s.entryCommand.bind( "<Return>",     s.__HandlerEntryCommand )
        s.entryCommand.bind( "<Up>",         s.__HandlerEntryKey )
        s.entryCommand.bind( "<Down>",       s.__HandlerEntryKey )

    # PRIVATE
    def __HandlerButtonRefresh( s ):
        """ Handle Refresh button.  Refresh according to which source files changed. """
        st.threads.Spawn( ThreadRefresh )
        
    # PRIVATE
    def __HandlerButtonClosePage( s ):
        """ Handle Close (Page) button.  Close current page in notebook """
        s.notebookSrcFile.CloseCurrentPage()
    
    # PRIVATE
    def __HandlerFilterSub( s, eParse, checkButton ):
        """ Subroutine to handle clicking a Filter checkbutton. """

        # Change parser mode (the parser does the filtering).
        if checkButton:
            st.parser.AddMode( eParse )
        else:
            st.parser.RemoveMode( eParse )
        st.log.Write( "filtering " + st.parser.GetModesString() )

        # Skip re-parsing/loading if notebook is empty.
        if not st.gui.notebookSrcFile.IfEmpty():
            st.gui.Enable( False )
            try:
                st.gui.statusbar.Print( "Filtering..." )
                # ReloadSrcFiles() leads to asking CacheSrcFile for
                # a ContentsSrcFile matching both fname and parserModes.
                # If not yet cached, the src file is read, tokenized, and cached.
                s.ReloadSrcFiles()
            except:
                PrintException()
            st.gui.Enable( True )

    # PRIVATE
    def __HandlerFilterComments( s ):
        """ Handle clicking a Filter checkbutton. """
        s.checkButtonFlagComments = not s.checkButtonFlagComments
        s.__HandlerFilterSub( eParseComments, s.checkButtonFlagComments )

    # PRIVATE
    def __HandlerFilterFunctions( s ):
        """ Handle clicking a Filter checkbutton. """
        s.checkButtonFlagFunctions = not s.checkButtonFlagFunctions
        s.__HandlerFilterSub( eParseFunctions, s.checkButtonFlagFunctions )

    # PRIVATE
    def __HandlerFilterData( s ):
        """ Handle clicking a Filter checkbutton. """
        s.checkButtonFlagData = not s.checkButtonFlagData
        s.__HandlerFilterSub( eParseData, s.checkButtonFlagData )

    # PRIVATE
    def __HandlerFileListbox( s, event ):
        """ Handle selection of a source file in listbox. """
        # Get item select in file listbox.
        try:
            fname = event.widget.get( "active" )
            st.gui.ScheduleIdleFunc( s.notebookSrcFile.LoadSrcFile, fname, 1 )
        except:
            PrintException()

    # PRIVATE
    def __HandlerFocusIn( s, event ):
        """ Handler for when a widget receives the focus. """
        try:
            if event.widget is s.entryCommand:
                s.labelCommand.config( bg=s.colorFocusIn )
        except:
            PrintException()

    # PRIVATE
    def __HandlerFocusOut( s, event ):
        """ Handler for when a widget loses the focus. """
        try:
            if event.widget is s.entryCommand:
                s.labelCommand.config( bg=s.colorFocusOut )
        except:
            PrintException()

    # PRIVATE
    def __HandlerEntryCommand( s, event ):
        """ Handle a command given by the user in entry widget. """
        try:
            command = s.entryCommand.get()
            s.entryCommand.delete( "0", TK.END )
            st.command.Do( command )
        except:
            PrintException()

    # PRIVATE
    def __HandlerProgramExit( s, event ):
        """ Handle a Tk Destroy (program exit) event. """
        # NOTE: All destroyed widgets will hit this point!
        # Therefore, ignore all of them except the root widget.
        if event.widget is st.gui.root:
            st.exiting = True

    # PRIVATE
    def __HandlerKeyPress( s, event ):
        """ Handle a key press in the main window (any window). """
        # -- TextWid.__HandlerKey() handles keyboard events in text widgets first --
        # This works only if pending Tk events are processed frequently enough.

        # Ignore overly rapid key events caused by keyboard repeat.
        if s.__IgnoreKeyEvent():
            return

        # Esc
        if event.keysym == "Escape":
            st.escKey = True
        # Ctrl-W
        elif event.keysym == "w":
            s.notebookSrcFile.CloseCurrentPage()
        # F1
        elif event.keysym == "F1":
            s.resultsHistory.Show( s.GetResultsTextWid() )
        # F2
        elif event.keysym == "F2":
            s.resultsHistory.ShowLast( s.GetResultsTextWid() )
        else:
            PrintWarning( "Gui: unhandled key press" )
                
    # PRIVATE
    def __HandlerKeyRelease( s, event ):
        """ Handle a key press in the main window (any window). """
        # -- TextWid.__HandlerKey() handles keyboard events in text widgets first --
        # This works only if pending Tk events are processed frequently enough.

        # Ignore overly rapid key events caused by keyboard repeat.
        if s.__IgnoreKeyEvent():
            return

        # Esc
        if event.keysym == "Escape":
            st.escKey = False

    # PRIVATE
    def __HandlerEntryKey( s, event ):
        """ Handle a key press in the command entry widget. """
        # Insert prior or next command in command history
        # back into the command entry widget.

        # (don't call __IgnoreKeyEvent(), let user type at a fast speed)
        
        try:
            if event.keysym in [ "Up", "Down" ]:
                if event.keysym == "Up":
                    command = st.command.Recall( prev=True )
                else:
                    command = st.command.Recall( next=True )
                if command:
                    s.entryCommand.delete( "0", TK.END )
                    s.entryCommand.insert( TK.END, command )
        except:
            PrintException()

    # PRIVATE
    def __IgnoreKeyEvent( s ):
        """ Return False which means keyboard events are happening too fast. """
        now  = time.time()
        last = s.keyEventTime
        if now - last < 1.0:            # one second delay between responses
            return True
        else:
            st.keyEventTime = now       # remember time of this allowed key event
            return False
                
#-------------------------------------------------------------------------------
# Class Statusbar.
#-------------------------------------------------------------------------------

# Updating of statusbar and threads:
# ----------------------------------
# The statusbar (as all widgets are) is rendered by the main thread.
# Long operations that run in the main thread will block updating the statusbar.
# If a long operation can be executed by another thread, the thread is allowed
# to call Statusbar.Print() which only changes its underlying string that
# is serialized by a mutex.  An idle time, run by the main thread, will update
# the statusbar widget with the changed string (statusbar text).
#
# Showing error/warnings on statusbar:
# ------------------------------------
# The statusbar is implemented as two Tk label widgets.
# Normal text is shown in one, and errors and warnings both are shown in the other.
# The Print*() methods (as explained above) just change a string
# and a timer function periodically updates the label widgets.
# Normal statusbar text times-out after a few seconds,
# but errors/warnings persist for a minute.

class Statusbar(StatusbarBase):

    # Thread-safe methods: (see Gui.py)

    def __init__( s, frame ):
        # Statusbar widget is lazy-updated by a timer.
        # Tk operations are slow.  Therefore, most calls to Statusbar.Print()
        # only affects a Python string.  Only once every timer is the Tk widget rendered.

        StatusbarBase.__init__( s, frame )

        s.text    = ""           # statusbar text
        s.textError = ""         # error/warning text
        s.textErrorPrev = ""     # previous
        s.textErrorPadding = ""
        if IfDarwin():           # Mac OS X: prevent bottom-right widget obscuring error
            s.textErrorPadding = "    "
        s.timeout = 0            # timeout of statusbar text
        s.timeoutError = 0       # timeout for error/warning text
        s.milsec  = 250          # frequency of timer
        s.changed = False        # set when Print() is called, cleared by timer
        s.batonChars = "-\|/"
        s.batonIdx = 0

        # Tk specific.  Don't confuse frame/s.frame.
        # Subtle Tk behavior: By creating the error label before the normal label,
        # if their text overlaps, error text will overlay normal text (as desired).
        s.frame = TK.Frame( frame, relief=TK.SUNKEN, borderwidth=2 )
        s.frame.pack( side=TK.BOTTOM, expand=TK.NO, fill=TK.BOTH )
        s.labelError = TK.Label( s.frame )
        s.labelError.pack( side=TK.RIGHT, expand=TK.NO, fill=TK.BOTH, anchor=TK.E )
        s.labelErrorColorOff = s.labelError["bg"]
        s.labelErrorColorOn  = "#d00000"
        s.label = TK.Label( s.frame )
        s.label.pack( side=TK.LEFT, expand=TK.NO, fill=TK.BOTH, anchor=TK.W )

        s.__RechargeTimer()

    # INTERFACE
    def Clear( s ):
        # This pends a change to statusbar text (rendering the widget is delayed).
        # -- don't call any GUI method, this can be run by second thread --
        try:
            st.gui.mutex.acquire()
            s.__Clear()
        finally:
            st.gui.mutex.release()

    # PRIVATE
    def __Clear( s ):
        # Clear statusbar text (not the actual widget itself).
        # -- don't call any GUI method, this can be run by second thread --
        s.changed = True
        s.text = ""

    # INTERFACE
    def Print( s, text, timeout=7000, baton=False, now=False ):
        """ Print text to statusbar. """
        # (see base)
        try:
            st.gui.mutex.acquire()
            # Allow only the main thread to execute in the GUI toolkit.
            if now and st.IfMainThread():
                # -- is NOT thread-safe: executes in GUI toolkit --
                s.text = Chomp(text)
                s.timeout = timeout
                s.__ChangeWidget( text )    # limited to main thread
                st.gui.ProcessPendingEvents()
            else:
                # -- is thread-safe --
                # A zero timeout arg means no timeout: change it to a a huge value.
                if timeout == 0:
                    timeout = 100*1000*1000
                s.text = Chomp(text)
                s.timeout = timeout
                s.changed = True
                if baton:
                    s.text += " " + s.batonChars[s.batonIdx]
                    s.batonIdx = (s.batonIdx + 1) % len(s.batonChars)
        finally:
            st.gui.mutex.release()

    # PRIVATE
    def __PrintError( s, text ):
        # Timer function will update the label widget.
        try:
            st.gui.mutex.acquire()
            # Change newlines otherwise label widget will expand vertically.
            s.textError = text.replace("\n"," ") + s.textErrorPadding
            s.timeoutError = 1000 * 60 * 1    # 1 minute as milliseconds
        finally:
            st.gui.mutex.release()

    # INTERFACE
    def PrintError( s, text ):
        """ Print error message. """
        s.__PrintError( "ERROR: " + text )

    # INTERFACE
    def PrintWarning( s, text ):
        """ Print warning message. """
        s.__PrintError( "WARNING: " + text )

    # PRIVATE
    def __Update( s ):
        """ Called by a timer (in main thread) to update/render statusbar widget. """

        st.gui.ProcessPendingEvents()

        # Update label widget showing any error/warning.
        s.__UpdateError()

        # Call an optional registered callback which is expected
        # to call Statusbar.Print() to set the text.
        # Can be used for periodically updating the statusbar every timer.
        if s.updateCallback is not None:
            s.updateCallback( *s.updateCallbackArgs )

        # Was Print() called recently?
        if s.changed:
            s.changed = False
            s.__ChangeWidget( s.text )
        # Clear only once when timeout becomes negative.
        elif s.timeout < 0 and s.text:
            s.__Clear()
            s.__ChangeWidget( "" )
        # Age statusbar text.
        s.timeout -= s.milsec
        s.__RechargeTimer()

    # PRIVATE
    def __UpdateError( s ):
        """ Update label widget with error/warning text. """
        if s.textError:
            # Age the text.
            s.timeoutError -= s.milsec
            # Clear the text after time-out.
            if s.timeoutError < 0:
                s.textError = ""
            # Change label widget if text changed.
            if s.textError != s.textErrorPrev:
                s.textErrorPrev = s.textError[:]    # copy
                # Change text.
                s.labelError.config( text=s.textError )
                # Change color.
                if s.textError:
                    s.labelError.config( bg=s.labelErrorColorOn )
                else:
                    s.labelError.config( bg=s.labelErrorColorOff )

    # PRIVATE
    def __ChangeWidget( s, text ):
        """ Directly set text of statusbar widget. """
        s.text = text
        s.label.config( text=text )

    # PRIVATE
    def __RechargeTimer( s ):
        st.gui.ScheduleTimerFunc( s.milsec, s.__Update )

#-------------------------------------------------------------------------------
# Class ScrollFrame.
#-------------------------------------------------------------------------------

class ScrollFrame(ScrollFrameBase):
    """ Create a frame and scrollbars for the given widget. """

    def __init__( s, parentWidget, WidgetClass, labelText, **args ):
        s.enabled = False
        s.frame = TK.Frame( parentWidget )

        if labelText:
            s.labelFrame = TK.Frame( s.frame )
            s.label = TK.Label( s.labelFrame, text=labelText )
            s.labelOrigColor = s.label["bg"]
        else:
            s.label = None
            s.labelOrigColor = None
        s.widget     = WidgetClass( s.frame, args )
        s.widget.ScrollFrame = s  # store reference back to its ScrollFrame container
        s.xscrollbar = TK.Scrollbar( s.frame, orient=TK.HORIZONTAL )
        s.yscrollbar = TK.Scrollbar( s.frame, orient=TK.VERTICAL   )
        
        if labelText:
            s.labelFrame.pack( side=TK.TOP, expand=TK.NO, fill=TK.BOTH, anchor=TK.W )
            s.label.pack( side=TK.LEFT, fill=TK.Y )
        s.yscrollbar.pack( side=TK.RIGHT,  fill=TK.Y )
        s.xscrollbar.pack( side=TK.BOTTOM, fill=TK.X )
        s.widget.pack( expand=TK.YES, fill=TK.BOTH )

        s.widget.config( xscrollcommand=s.xscrollbar.set, yscrollcommand=s.yscrollbar.set )
        s.xscrollbar.config( command=s.widget.xview )
        s.yscrollbar.config( command=s.widget.yview )

    # INTERFACE
    def Enable( s, enable=True ):
        s.enabled = enable
        st.gui.EnableWidget( s.widget, enable )
        if s.label is not None:
            st.gui.EnableWidget( s.label, enable )

    # INTERFACE
    def SetLabelText( s, text ):
        """ Set the text of a label widget in a scroll-frame widget. """
        try:
            if s.label:
                s.label.config( text=text )
        except:
            PrintException()

    # INTERFACE
    def SetLabelColor( s, color ):
        """ Set the color of a label widget in a scroll-frame widget. """
        # Pass None to restore original color.
        try:
            if not color:
                color = s.labelOrigColor
            s.label.config( bg=color )
        except:
            PrintException()

    # INTERFACE
    def GetScrollbarPos( s ):
        """ Returns (x,y) tuple of horizontal and vertical scrollbars. """
        (x,dontcare) = s.xscrollbar.get()
        (y,dontcare) = s.yscrollbar.get()
        return (x,y)

    # INTERFACE
    def SetScrollbarPos( s, xy ):
        """ Set position of horizontal and vertical scrollbars (xy is a tuple). """
        if xy is not None:
            #x = xy[0]   # ignore horizonal position (unimportant)
            y = xy[1]
            if y is not None:  
                s.widget.yview_moveto( y )

#-------------------------------------------------------------------------------
# Class NotebookPage.
#-------------------------------------------------------------------------------

# Page names in Pmw notebooks:
# - Outside code gives a name to a page.
#   Given names are associated with a unique name (a stringified int count).
#   The unique name, not the given name, is passed down to Pmw when the page is created.
#   The reason is Pmw page names must be unique and can't be renamed
#   Notebook.__RenamePage() effectively renames a page can changing the given name
#   (re-associating the same unique name but with a different given name).
# - Page names are not the same as its tab text (tab text is for display)
#   as pypersrc could load files with the same names but different paths.
#   The Pmw NoteBook class also treats page names and tab text as different.

class NotebookPage(object):

    nextPageId = 0

    def __init__( s, pageName, tabText, notebookWidget, append=True ):
        # append arg is ignored (always appends)
        s.pageName       = pageName         # given page name
        s.notebookWidget = notebookWidget
        s.pageId         = s.__NewPageId()  # allocate a unique ID for Pmw page widget.
        s.pageWidget     = s.__NewPageWidget( s.pageId, tabText )
        s.SetTabText( tabText )

    # INTERFACE
    def GetPageName( s ):
        """ Return page's given name. """
        # This is the name of the page given by pypersrc (as opposed to page ID used by Pmw).
        return s.pageName
    
    # INTERFACE
    def GetPageId( s ):
        """ Return page ID. """
        # pageId is how the page is known to Pmw (pypersrc knows pages by given names).
        return s.pageId
    
    # INTERFACE
    def GetPageWidget( s ):
        """ Return page widget. """
        return s.pageWidget
        
    # INTERFACE
    def GetTabWidget( s ):
        """ Return tab widget (part of this page). """
        # tab() is a Pmw Notebook method.
        tabWidget = s.notebookWidget.tab( s.pageId )
        return tabWidget

    # INTERFACE
    def RenamePage( s, newPageName ):
        """ Change given name of page (page ID unchanged). """
        s.pageName = newPageName
        s.SetTabText( s.pageName )

    # INTERFACE
    def SetTabText( s, tabText ):
        """ Set text in tab widget of page. """
        # Pmw has a known bug where changing tab text doesn't change tab width
        # (not much of a problem for pypersrc).
        tabWidget = s.GetTabWidget()
        tabWidget["text"] = tabText      # implicit call to Pmw/Tkinter method
        
    # PRIVATE
    def __NewPageId( s ):
        """ Allocate a unique page ID for a new page. """
        NotebookPage.nextPageId += 1
        # Must stringify the page ID to pass to Pmw.
        return str(NotebookPage.nextPageId)

    # PRIVATE
    def __NewPageWidget( s, pageId, tabText ):
        # Return new page widget appended to Pmw notebook widget.
        # Pmw NoteBook.insert()/add() returns a page widget.
        return s.notebookWidget.add( pageId, tab_text=tabText )

#-------------------------------------------------------------------------------
# Class Notebook (Pmw).
#-------------------------------------------------------------------------------

# Pmw widgets inherit many methods from the Tkinter widgets.

class Notebook(NotebookBase):
    """ Generalized (but Tk-specific) class for a notebook widget. """
    # Functionality deferred to derived class:
    # - loading the contents of a page
    # - enable widgets contained in pages

    def __init__( s, parentWidget ):
        # Create a Pmw notebook widget.
        s.notebookWidget = Pmw.NoteBook( parentWidget, raisecommand=s.__HandlerPageSelected )
        s.notebookWidget.pack( fill="both", expand=1, padx=0, pady=0 )

        # Maps given page names to NotebookPage objects.
        s.pageDict = {}    # givenPageName : NotebookPage object

    # INTERFACE
    def Enable( s, enable ):
        """ This enables only the notebook and pages themselves.
            A derived class must enable whatever widgets contained in its pages. """
        # Enable notebook widget.
        st.gui.EnableWidget( s.notebookWidget, enable )

        # Enable all pages.
        #for pageWidget in s.GetListPageWidget():         # doesn't work as Pmw page widget
        #    st.gui.EnableWidget( pageWidget, enable )    # evidently doesn't implement "state"
        
        # Enable all tab widgets of each page.
        for tabWidget in s.GetListTabWidget():
            st.gui.EnableWidget( tabWidget, enable )

    # INTERFACE
    def IfEmpty( s ):
        """ Return true if no pages. """
        return len(s.pageDict) == 0

    # INTERFACE
    def AddPage( s, pageName, tabText, append=True ):
        """ Add a new notebook page.  Given page name must be unique.  tabText is for display.
            Returns widget so that caller can pack a child widget into it. """
        page = NotebookPage( pageName, tabText, s.notebookWidget, append )
        s.pageDict[pageName] = page        # associate NotebookPage object
        s.__UpdateClosePageButton()        # to enable button when there's at least one page
        return page.GetPageWidget()        # return Pmw page widget

    # INTERFACE
    def RemovePage( s, pageName ):
        """ Remove a notebook page. """
        page = s.pageDict[pageName]        # get NotebookPage object
        pageId = page.GetPageId()          # pageId is how page is known to Pmw
        s.notebookWidget.delete( pageId )  # call Pmw method
        del s.pageDict[pageName]
        s.__UpdateClosePageButton()        # to enable button when there's at least one page

    # PROTECTED
    def RenamePage( s, oldPageName, newPageName ):
        page = s.pageDict[oldPageName]
        s.page.RenamePage( newPageName )

    # INTERFACE
    def SelectPage( s, pageName ):
        """ Select a page. """
        page = s.pageDict[pageName]
        pageId = page.GetPageId()
        s.notebookWidget.selectpage( pageId )

    # INTERFACE
    def GetCurrentPageName( s ):
        """ Return name of current page or None if no pages exist. """
        try:
            pageId = s.notebookWidget.getcurselection()
            pageName = s.__PageId2PageName( pageId )
            return pageName
        except:
            return None

    # INTERFACE
    def CloseCurrentPage( s ):
        """ Close the current page. """
        pageName = s.GetCurrentPageName()
        if pageName:
            s.RemovePage( pageName )

    # PROTECTED
    def SetTabText( s, pageName, tabText ):
        """ Set text in tab widget of page. """
        page = s.pageDict[pageName]
        page.SetTabText( tabText )
        
    # PROTECTED
    def GetListPageWidget( s ):
        """ Return list of page widgets. """
        l = []
        for page in s.pageDict.values():
            l.append( page.GetPageWidget() )
        return l

    # PROTECTED
    def GetListTabWidget( s ):
        l = []
        """ Return list of tab widgets (part of page). """
        for page in s.pageDict.values():
            l.append( page.GetTabWidget() )
        return l
    
    # PRIVATE
    def __HandlerPageSelected( s, pageId ):
        # Reflect selected notepage page in titlebar.
        try:
            pageName = s.__PageId2PageName( pageId )
            st.gui.SetTitle( pageName )
        except:
            # An exception will normally occur while a page is being added because
            # this point is hit before pypersrc has mapped page IDs with page names.
            pass

    # PRIVATE
    def __PageId2PageName( s, pageId ):
        """ Convert page ID to a given page name. """
        # Converts a Pmw page name to a pypersrc page name.
        for page in s.pageDict.values():
            if pageId == page.GetPageId():
                return page.GetPageName()
        raise IndexError

    # PRIVATE
    def __UpdateClosePageButton( s ):
        # Gray-out "Close (Page)" button when the notebook is empty.
        if s.IfEmpty():
            st.gui.DisableWidget( st.gui.buttonClosePage )
            st.gui.SetTitle( "" ) # and clear titlebar
        else:
            st.gui.EnableWidget( st.gui.buttonClosePage )

#-------------------------------------------------------------------------------
# Class NotebookTextWid (Pmw).
#-------------------------------------------------------------------------------

class NotebookTextWid(Notebook,NotebookTextWidBase):

    def __init__( s, parentWidget ):
        Notebook.__init__( s, parentWidget )
        s.textWidDict = {}   # givenPageName : textWid

    # INTERFACE
    def Enable( s, enable ):
        """ This enables the notebook, pages, and the text widgets contained in the pages. """
        # Base method enables all widgets except the widgets contained in pages.
        Notebook.Enable( s, enable )
        # Enable all the text widgets contained in pages.
        textWidList = s.GetTextWidList()
        for textWid in textWidList:
            textWid.Enable( enable )
            
    # INTERFACE
    def LoadContents( s, contents, line=0, hiliteLine=False, currentPage=False, selectPage=True ):
        """ Load notebook page with contents.  Updates title bar. """
        # currentPage can be True even if notebook is empty
        # in which case the first notebook page will be added now.

        pageName = contents.fname
        tabText  = Basename(contents.fname)               # abbrev fname

        # Determine which text widget to load.
        # Replace current page?
        if currentPage and s.GetCurrentTextWid():         # unless notebook has no page yet
            oldPageName = s.CurrentPageName()
            newPageName = pageName
            s.RenamePage( oldPageName, newPageName )
            textWid = s.GetCurrentTextWid()
        # Is a notebook page holding it?
        elif s.textWidDict.has_key( pageName ):
            textWid = s.textWidDict[pageName]
        else: # load in new page
            s.AddPage( pageName, tabText )                # creates notebook page and text widget
            textWid = s.textWidDict[pageName]
            
        # Load text widget.
        textWid.LoadContents( contents=contents, line=line, hiliteLine=hiliteLine )
        if selectPage:
            s.SelectPage( pageName )
            st.gui.SetTitle( pageName )

    # INTERFACE
    def AddPage( s, pageName, tabText, append=True ):
        """ Add a new notebook page.  Given page name must be unique.  tabText is for display.
            Returns widget so that caller can pack a child widget into it. """
        # This derived method additionally manages textWidDict.
        pageWidget = Notebook.AddPage( s, pageName, tabText ) # create page widget
        textWid = TextWidEditor( pageWidget ) # create text widget (inside page)
        s.textWidDict[pageName] = textWid     # remember new page in dict

    # INTERFACE
    def RemovePage( s, pageName ):
        """ Remove a notebook page. """
        # This derived method additionally manages textWidDict.
        Notebook.RemovePage( s, pageName )
        del s.textWidDict[pageName]           # forget destroyed text widget

    # INTERFACE
    def GetCurrentTextWid( s ):
        """ Return current text widget or None if no pages exist. """
        try:
            name = s.GetCurrentPageName()
            return s.textWidDict[name]
        except:
            return None    # else notebook is empty
        
    # INTERFACE
    def GetTextWidList( s ):
        """ Return list of text widgets in notebook. """
        textWidList = s.textWidDict.values()
        return textWidList
    
#-------------------------------------------------------------------------------
# Class NotebookSrcFile (Pmw).
#-------------------------------------------------------------------------------

class NotebookSrcFile(NotebookTextWid,NotebookSrcFileBase):
    """ Notebook class specialized for src files. """

    def __init__( s, parentWidget ):
        NotebookTextWid.__init__( s, parentWidget )

    # INTERFACE
    def LoadSrcFile( s, fname, line=0, hiliteLine=False, currentPage=False, selectPage=True ):
        """ Load src file into notebook page. """
        # Subtle: If the same fname is passed, different contents could be loaded
        # if CacheSrcFile was flushed or the parser is now in a different mode (filtering changed).
        try:
            st.gui.statusbar.Print( "Loading " + fname + "...", now=True, timeout=0 )
            st.gui.Enable( False )    # producing and loading contents may take a while
            try:
                # CacheSrcFile will read src file if it isn't cached.
                contents = st.gui.cacheSrcFile.Get( fname, st.parser.GetModes() )
                # Load notebook page with src file.
                s.LoadContents( contents, line, hiliteLine, currentPage, selectPage )
            except IOError:
                PrintWarning( "could not read source file '" + fname + "'" )
                return
        finally:
            st.gui.Enable( True )
        
    # INTERFACE
    def Reload( s ):
        """ Reload src files (shown in every notebook page). """
        textWidList = s.GetTextWidList()
        for textWid in textWidList:    # for each text widget in notebook
            # Subtle: Although the same fname is fed-back, different contents could be loaded
            # if CacheSrcFile was flushed or the parser is now in a different mode (filtering changed).
            fname = textWid.contents.fname
            s.LoadSrcFile( fname=fname, selectPage=False )
