# $Id: GuiTkText.py,v 1.25 2006-03-26 03:03:32 devel Exp $
# pypersrc
# Author:   Jim Brooks  http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# Notes:    - Tk Text widgets are inserted with "tags" (not to be confused with src code tags).
#             tktag is written to disambiguate.
#           - Tk will combine similar tags.
#           - Disabling a Tk Text widget isn't the desired way to turn in read-only.
#             When disabled, it will ignore keyboard input (read-only), but
#             disabling causes methods, such as insert() and delete(), to become NOPs.
#           - Read-only Tk Text widgets are simulated using the trick of installing
#             a keyboard handler that returns "break" to stop propagation of events.
#             This trick allows the widget effectively read-only but still be enabled
#             (state=normal), so that insert()/delete() will work.
#           - Historical note: Text widgets used to be disabled to turn them read-only.
#             But a disabled text widget causes several problems: methods such as insert()
#             become NOPs (requiring the kludge of temporarily enabling the widget),
#             scrollbar widgets ignore Page Up/Down keys, and can't receive the focus.
#           - See GuiText.py.
# ==============================================================================

#-------------------------------------------------------------------------------
# Class TextWid.
#-------------------------------------------------------------------------------

class TextWid(ScrollFrame,TextWidBase):
    """ Text widget contained in a scroll-frame widget. """

    def __init__( s, frame ): # frame (not fname)

        # Constructor args are distributed to multiple-inherited base classes.
        TextWidBase.__init__( s )
        ScrollFrame.__init__( s, frame, TK.Text, " ", wrap=TK.NONE )
        s.frame.pack( side=TK.TOP, expand=TK.YES, fill=TK.BOTH )
        s.__InstallHandlers()
        s.widget.TextWid = s            # let widget have a reference back to its TextWid class
        s.enabled        = False

        # Fonts.
        if st.gui.fontName and st.gui.fontSize:
            font = tkFont.Font( family=st.gui.fontName, size=st.gui.fontSize )
        elif st.gui.fontName:
            font = tkFont.Font( family=st.gui.fontName )
        elif st.gui.fontSize:
            font = tkFont.Font( size=st.gui.fontSize )
        else:
            font = None
        if font:
            s.widget.tag_config( st.gui.TKTAG_DEFAULT, font=font, foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_TEXT,    font=font, foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_NUM,     font=font, foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_SYMBOL,  font=font, foreground="darkred" )
            s.widget.tag_config( st.gui.TKTAG_WORD,    font=font, foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_KEYWORD, font=font, foreground="darkblue" )
            s.widget.tag_config( st.gui.TKTAG_TYPE,    font=font, foreground="purple" )
            s.widget.tag_config( st.gui.TKTAG_LOUD,    font=font, foreground="gold2" )
            s.widget.tag_config( st.gui.TKTAG_QUOTED,  font=font, foreground="#804040" )
            s.widget.tag_config( st.gui.TKTAG_COMMENT, font=font, foreground="darkgreen" )
            s.widget.tag_config( st.gui.TKTAG_TAGDEF,  font=font, foreground="darkcyan" )
            s.widget.tag_config( st.gui.TKTAG_TAGREF,  font=font, foreground="darkcyan", underline=1 )
            s.widget.tag_config( st.gui.TKTAG_TAGREFV, font=font, foreground="#a000a0", underline=1 )
        else:
            s.widget.tag_config( st.gui.TKTAG_DEFAULT,            foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_TEXT,               foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_NUM,                foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_SYMBOL,             foreground="darkred" )
            s.widget.tag_config( st.gui.TKTAG_WORD,               foreground="black"  )
            s.widget.tag_config( st.gui.TKTAG_KEYWORD,            foreground="darkblue" )
            s.widget.tag_config( st.gui.TKTAG_TYPE,               foreground="purple" )
            s.widget.tag_config( st.gui.TKTAG_LOUD,               foreground="gold2" )
            s.widget.tag_config( st.gui.TKTAG_QUOTED,             foreground="#804040" )
            s.widget.tag_config( st.gui.TKTAG_COMMENT,            foreground="darkgreen" )
            s.widget.tag_config( st.gui.TKTAG_TAGDEF,             foreground="darkcyan" )
            s.widget.tag_config( st.gui.TKTAG_TAGREF,             foreground="darkcyan", underline=1 )
            s.widget.tag_config( st.gui.TKTAG_TAGREFV,            foreground="#a000a0", underline=1 )

    # PRIVATE
    def __InstallHandlers( s ):
        """ Install handlers for TextWid. """
        s.widget.tag_bind( st.gui.TKTAG_TAGREF,  "<Button-1>", s.__HandlerHyperlink1 )
        s.widget.tag_bind( st.gui.TKTAG_TAGREF,  "<Button-2>", s.__HandlerHyperlink2 )
        s.widget.tag_bind( st.gui.TKTAG_TAGREF,  "<Button-3>", s.__HandlerHyperlink3 )
        s.widget.tag_bind( st.gui.TKTAG_TAGREFV, "<Button-1>", s.__HandlerHyperlinkVisited1 )
        s.widget.tag_bind( st.gui.TKTAG_TAGREFV, "<Button-2>", s.__HandlerHyperlinkVisited2 )
        s.widget.tag_bind( st.gui.TKTAG_TAGREFV, "<Button-3>", s.__HandlerHyperlinkVisited3 )
        s.widget.bind( "<FocusIn>",  s.__HandlerFocusIn )
        s.widget.bind( "<FocusOut>", s.__HandlerFocusOut )
        s.widget.bind( "<Key>", s.__HandlerKey )

    # INTERFACE
    def Enable( s, enable=True ):
        """ Enable TextWid. """
        ScrollFrame.Enable( s, enable )
        st.gui.EnableWidget( s.widget, enable )

    # INTERFACE
    def LoadContents( s, contents, line=0, hiliteLine=False, scrollbarPos=None ):
        """ Load text widget with a Contents object. """
        # This wrapper temporarily disables the GUI while loading Tk text widget which can be slow.
        st.gui.statusbar.Print( "Loading " + contents.fname + "...", now=True, timeout=0 )
        st.gui.Enable( False )      # disable entire GUI
        s.Enable()                  # but enable text widget or it can't be changed
        s.__LoadContents( contents, line, hiliteLine, scrollbarPos )
        st.gui.Enable( True )       # enable GUI (and text widget)
        st.gui.statusbar.Clear()

    # PRIVATE
    def __LoadContents( s, contents, line=0, hiliteLine=False, scrollbarPos=None ):
        # PRECOND: Text widget is enabled (otherwise insert()/delete() would be NOPs).
        # - contents : To implicitly force a reload, pass a different Contents object.
        # - line : Lines are counted from 1.  If line > 0, text widget is scrolled to line.
        # - scrollbarPos : (x,y) tuple (or None) of positions of horizontal,vertical scrollbars

        s.contents = contents

        s.widget.delete( s.__Idx(1), TK.END )    # this can be slow, widget state must be enabled prior

        # Load text widget from tokens using Tk's tag mechanism.
        # Tk tag lists are cached as a member of the Contents class,
        # but initially the member doesn't exist.
        try:
            # Executes if before Tk tag list was computed.
            tktagList = s.contents.tktagList    # possible exception
            st.log.Write( "TextWid: loading " + s.contents.fname + " (have Tk tags)" )
        except:
            # Compute Tk tag list, store/cache it as a class member of Contents.
            tktagList = s.Contents2TktagList( s.contents )
            s.contents.tktagList = tktagList
            st.log.Write( "TextWid: loading " + s.contents.fname + " (built Tk tags)" )

        # Insert Tk tags into text widget.
        for tktag in tktagList:
            text   = tktag[0]
            tktag2 = tktag[1]
            s.widget.insert( TK.END, text, (tktag2) )   # widget state must be enabled prior

        # Change position of scrollbars.
        if scrollbarPos is not None:
            s.SetScrollbarPos( scrollbarPos )

        s.SetLabelText( s.contents.fname )
        s.GotoLine( line, hiliteLine )   # NOP if line=0
        
    # PROTECTED/ABSTRACT
    def Contents2TktagList( s, contents ):
        # If a source file is being loaded, all token tuples are one type:
        # (eToken*,text)
        # Otherwise, there are two types of token tuples.
        # The phony tag tuple is used to show hyperlinks in the results
        # of user commands such as find text.
        # (eTokenTag,(PhonyTag))
        # (eTokenText,text)
        raise Oops( "abstract method, derived TextWid must define this!" )

    # INTERFACE
    def GotoLine( s, line, hiliteLine=False ):
        # Ignore if line=0.
        if line >= 1:
            # Scroll widget to line.
            s.widget.see( s.__Idx(line) )
            s.widget.mark_set( TK.INSERT, s.__Idx(line) )

            # Hilite line.
            if hiliteLine:
                try:
                    s.widget.tag_delete( "PYMATCH" )    # forget previous matching lines
                except:
                    PrintException()
                s.widget.tag_add( "PYMATCH", s.__Idx(line), s.__Idx(line,999) )    # column 0...999
                s.widget.tag_config( "PYMATCH", background="cyan" )
        
            # Prevent an unintentional text selection during scrolling
            # (can happen if user has mouse button down while this is executing).
            s.widget.tag_remove( TK.SEL, s.__Idx(1), TK.END )

    # PRIVATE
    def __Idx( s, line, col=0 ):
        # Lines are counted from 0.
        # Tk-specific: Return an Tk Text widget index for location at line and column.
        # The return value can be passed to Tk Text.insert() eg.
        return "%d.%d" % (line,col)

    # PROTECTED
    def Clear( s, tokenList=[], fname=None ):
        """ Clear text widget. """
        # fname can be used as the title (for display).
        contents = ContentsNone( tokenList=tokenList, fname=fname )
        s.LoadContents( contents=contents )

    # PRIVATE
    def __SetTktagList( s, key, val ):
        i = 0
        for tktag in s.contents.tktagList:
            if tktag[0] == key:
                s.contents.tktagList[i] = (key,val)
            i += 1

    # PRIVATE
    def __HandlerFocusIn( s, event ):
        # If text widget loaded, change label to a brighter color.
        textWid = event.widget.TextWid
        if not textWid.IfEmpty():
            textWid.SetLabelColor( st.gui.colorFocusIn )

    # PRIVATE
    def __HandlerFocusOut( s, event ):
        # Focus has left, change label back to default color.
        textWid = event.widget.TextWid
        textWid.SetLabelColor( None )

    # PRIVATE
    def __HandlerKey( s, event ):
        # Trick to simulate a read-only-but-enabled Tk text widget.
        # Discard keys that would modify the text widget by returning "break".
        # But allow movement keys events to propagate scrollbar widget.
        allowed = event.keysym in [ "Escape", "Next", "Prior", \
                                    "Up", "Down", "Left", "Right", \
                                    "Home", "End", "Control_L", "Control_R", \
                                    "F1", "F2", "F3", "F4", "F5", "F6", \
                                    "F7", "F8", "F9", "F10", "F11", "F12" ]
        allowed = allowed or event.char in [ "\x17" ]    # Ctrl-W = 0x17
        # If not a movement key, assume the key would modify the text widget.
        if not allowed and not s.contents.editable:
            st.gui.statusbar.Print( "Text widget is read-only (pypersrc is not an editor)." )
            return "break"    # stop event
        else:
            return            # let event propagate

    # PRIVATE
    def __HandlerHyperlink1( s, event ):
        s.__HandlerHyperlinkSub( event, st.gui.TKTAG_TAGREF, 1 ) # button 1

    # PRIVATE
    def __HandlerHyperlink2( s, event ):
        s.__HandlerHyperlinkSub( event, st.gui.TKTAG_TAGREF, 2 ) # button 2

    # PRIVATE
    def __HandlerHyperlink3( s, event ):
        s.__HandlerHyperlinkSub( event, st.gui.TKTAG_TAGREF, 3 ) # button 3

    # PRIVATE
    def __HandlerHyperlinkVisited1( s, event ):
        s.__HandlerHyperlinkSub( event, st.gui.TKTAG_TAGREFV, 1 ) # button 1

    # PRIVATE
    def __HandlerHyperlinkVisited2( s, event ):
        s.__HandlerHyperlinkSub( event, st.gui.TKTAG_TAGREFV, 2 ) # button 2

    # PRIVATE
    def __HandlerHyperlinkVisited3( s, event ):
        s.__HandlerHyperlinkSub( event, st.gui.TKTAG_TAGREFV, 3 ) # button 3

    # PRIVATE
    def __HandlerHyperlinkSub( s, event, tktag, button ):
        """ Handle clicking hyperlink in a text widget. """
        # See GuiText.py for mouse button behavior.
        # tkag is TKTAG_TAGREF or TKTAG_TAGREFV.
        # .................................................
        # Button 1 means to load into a new notebook page.
        # Button 2 means to load the current notebook page.
        widget = event.widget
        currentPage = (button != 1)
        try:
            textWid = widget.TextWid
            # This might be slow on older TK versions (eg Redhat 8).
            st.gui.statusbar.Print( "Locating hyperlink..." )
            # Find the nearest Tk tag to where the user clicked.
            pos = '@%d,%d' % (event.x+1, event.y)
            (pos1,pos2) = widget.tag_prevrange( tktag,  pos )
            tag = widget.get( pos1, pos2 ) # this a Tk tag but may also be a source code tag (confusing)
            if isinstance(textWid.contents,ContentsResults):
                # The user clicked a hyperlink in a results text widget.
                # Search the results tokenList for what was clicked.
                # token[1] holds a phony tag (special case).
                for token in textWid.contents.tokenList:
                    if st.tags.IfPhonyTagInToken( token ):
                        phonyTag = token[1]
                        if tag == phonyTag[TAG_NAME]:
                            # Immediately change the color of the hyperlink
                            # to indicate that it was visited (dark purple).
                            widget.delete( pos1, pos2 )
                            widget.insert( pos1, tag, (st.gui.TKTAG_TAGREFV) )

                            # -- This affects cached ContentsResults --
                            # Update Tk tag list.
                            s.__SetTktagList( tag, st.gui.TKTAG_TAGREFV )

                            # -- This affects non-cached ContentsResults (eg ContentsHistory) --
                            # Remember that this hyperlink was visited by the user.
                            # This causes the TextWid.LoadText() to color the hyperlink
                            # as dark purple if the results are reloaded from the cache.
                            st.visited[tag] = True

                            fname = phonyTag[TAG_FILE]
                            line  = phonyTag[TAG_LINE]
                            if isinstance(textWid.contents,ContentsHistory):
                                # Hyperlink clicked in the history of command results.
                                st.gui.resultsHistory.Load( st.gui.GetResultsTextWid(), fname )
                            else:
                                # Load into current or different notebook page (text widget).
                                st.gui.notebookSrcFile.LoadSrcFile( fname, line=line, hiliteLine=True, currentPage=currentPage )
                            break
            else:
                st.gui.TagClick( clickedTagName=tag, clickedFname=textWid.contents.fname )
        except:
            PrintException()
        st.gui.statusbar.Clear()
                
#-------------------------------------------------------------------------------
# Class TextWidEditor.
#-------------------------------------------------------------------------------

# TextWidEditor doesn't mean a text widget that is an editor.
# Rather, it has buttons for running an external editor (emacs/vi).

class TextWidEditor(TextWid):
    """ Specialized text widget with buttons for invoking external editor (emacs/vi). """

    def __init__( s, frame ): # frame (not fname)
        TextWid.__init__( s, frame )
        # Create editor buttons.
        if st.editorXterm:
            s.buttonEditorXterm = TK.Button( s.labelFrame, text=Basename(st.editorXterm), command=s.__HandlerButtonEditorXterm )
            s.buttonEditorXterm.pack( side=TK.RIGHT, anchor=TK.E )
            st.gui.DisableWidget( s.buttonEditorXterm )
        if st.editorGui:
            s.buttonEditorGui = TK.Button( s.labelFrame, text=Basename(st.editorGui), command=s.__HandlerButtonEditorGui )
            s.buttonEditorGui.pack( side=TK.RIGHT, anchor=TK.E )
            st.gui.DisableWidget( s.buttonEditorGui )

    # INTERFACE
    def Enable( s, enable=True ):
        """ Enable TextWid. """
        TextWid.Enable( s, enable )
        if st.editorGui:
            st.gui.EnableWidget( s.buttonEditorGui, enable )
        if st.editorXterm:
            st.gui.EnableWidget( s.buttonEditorXterm, enable )

    # INTERFACE
    def LoadContents( s, contents, line=0, hiliteLine=False, scrollbarPos=None ):
        """ Load text widget with a Contents object. """
        # Jump to another line in the same contents?
        if (contents is s.contents) and (not scrollbarPos):
            s.GotoLine( line, hiliteLine )
        else:
            TextWid.LoadContents( s, contents, line, hiliteLine, scrollbarPos )

    # PROTECTED
    def Contents2TktagList( s, contents ):
        """ Convert a Contents object to a list of Tk text tags. """
        # (see base method for comments)
        tktagList  = []
        line       = 1
        fname      = contents.fname
        tokenList  = contents.tokenList
        for token in tokenList:
            eToken = token[0]    # token enum
            text   = token[1]
            # Underline a reference to source code tags
            # to make a HTML-like hyperlink but not for
            # a source tag on the line where it's defined.
            if eToken == eTokenTag:
                if st.tags.IfTagRef( text, fname, line ):
                    tktag = st.gui.TKTAG_TAGREF
                else:
                    tktag = st.gui.TKTAG_TAGDEF
            else:
                tktag = st.gui.tktagDict[eToken]
            tktagList.append( (text,tktag) )
            line += text.count( "\n" )
        return tktagList
    
    # INTERFACE
    def Refresh( s, change, addedList, modifiedList, removedList ):
        # Pass change=False if no source files/dirs really changed,
        # in which case only the log will be refreshed if it's showing.

        # Return if nothing needs refreshing.
        if not change:
            return

        def RefreshReloadTextwid( textwid ):
            # PRECOND: The src file was flushed out of the cache.
            # But still ask the cache for the src file in order to re-cache it.
            fname = textwid.contents.fname
            contents = st.gui.cacheSrcFile.Get( fname, st.parser.GetModes() )
            textwid.LoadContents( contents=contents )    # assigns s.contents

        # Was the src file (being shown) modified?
        if s.contents.fname in modifiedList:
            st.log.Write( "TextWid: refreshing " + s.contents.fname )
            RefreshReloadTextwid( s )
            return
            
        # Was the src file (being shown) deleted?
        if s.contents.fname in removedList:
            # Set .fnamedSaved so that the file can be reloaded
            # by refresh if later reinstated by the user.
            tokenList = [ (eTokenText,"("+s.contents.fname+" no longer exists)") ]
            s.Clear( tokenList=tokenList, fname=s.contents.fname )
            s.contents.missing    = True     # remember that it was removed externally
            s.contents.fnameSaved = s.contents.fname
            return
            
        # Consequently, was the source file deleted but later reinstated?
        if s.contents.fname in addedList and s.contents.missing:
            st.log.Write( "TextWid: refreshing reinstated '" + s.contents.fnameSaved + "'" )
            RefreshReloadTextwid( s )
            s.contents.missing    = False
            s.contents.fnameSaved = None
            return
            
        # No reason to refresh this text widget.
        return

    # PRIVATE
    def __HandlerButtonEditorGui( s ):
        """ Handler for starting a native GUI editor (eg xemacs,gedit). """
        if st.editorGui:
            prog      = st.editorGui
            argsTuple = (st.editorGui, s.contents.fname)
            s.__HandlerButtonEditor( prog, st.editorGui, argsTuple )

    # PRIVATE
    def __HandlerButtonEditorXterm( s ):
        """ Handler for starting an editor that must run in an xterm (eg vi/vim). """
        if st.editorXterm:
            prog      = "xterm"
            argsTuple = ("xterm", "-e", st.editorXterm, s.contents.fname)
            s.__HandlerButtonEditor( prog, st.editorXterm, argsTuple )

    # PRIVATE
    def __HandlerButtonEditor( s, prog, editor, argTuple ):
        # prog:   program to run (could be the editor itself or xterm)
        # editor: name of editor
        try:
            if prog:
                st.log.Write( "running editor " + editor + " on " + s.contents.fname )
                RunProgram( prog, True, argTuple )
            else:
                PrintWarning( "Gui: editor not specified" )
        except:
            PrintException()
    
#-------------------------------------------------------------------------------
# Class TextWidResults.
#-------------------------------------------------------------------------------

class TextWidResults(TextWid):
    """ Specialized text widget for holding results. """

    def __init__( s, frame ): # frame (not fname)
        TextWid.__init__( s, frame )
        # To keep log refreshed if it is being shown.
        s.logLen        = 0
        s.milsecIdleLog = 2000
        st.gui.ScheduleTimerFunc( s.milsecIdleLog, s.__IdleLog )
        
    # INTERFACE
    def LoadContents( s, contents, line=0, hiliteLine=False, scrollbarPos=None ):
        """ Load text widget with a Contents object. """
        # Store outgoing results in history.
        st.gui.resultsHistory.Save( s )
        TextWid.LoadContents( s, contents, line, hiliteLine, scrollbarPos )

    # PROTECTED
    def Contents2TktagList( s, contents ):
        """ Convert a Contents object to a list of Tk text tags. """
        # (see base method for comments)
        tktagList  = []
        line       = 1
        fname      = contents.fname
        tokenList  = contents.tokenList
        for token in tokenList:
            # Is this a (eTokenTag,(PhonyTag)) ?
            if st.tags.IfPhonyTagInToken( token ):
                phonyTag = token[1]
                text = phonyTag[TAG_NAME]
                # Display visited and not-yet-visited hyperlinks
                # in different colors (like a web browser would do).
                # These two Tk tags are associated with different colors.
                if st.visited.has_key( text ):
                    tktag = st.gui.TKTAG_TAGREFV
                else:
                    tktag = st.gui.TKTAG_TAGREF
                tktagList.append( (text,tktag) )
                # Insert different tag to prevent Tk from combining like tags.
                # An empty line between matches makes clicking easier for the user.
                tktagList.append( ("\n\n",st.gui.TKTAG_DEFAULT) )
            else:
                # No, this is a usual (eToken*,text).
                text = token[1]
                tktag = st.gui.TKTAG_DEFAULT
                tktagList.append( (text,tktag) )
        return tktagList
    
    # INTERFACE
    def Refresh( s, change, addedList, modifiedList, removedList ):
        # Pass change=False if no source files/dirs really changed,
        # in which case only the log will be refreshed if it's showing.

        # Is this text widget showing results?
        # If so, rerun its command if it is affected by the source file change.
        # But always refresh the log if it's being shown.
        if change and (not isinstance(s.contents,ContentsNone)) and s.contents.command: # order-dependent
            st.log.Write( "refresh: redoing command '" + s.contents.command + "'" )
            s.contents.UpdateRefreshCnt()
            st.command.Do( s.contents.command )
        else:
            # Refresh unnecessary.
            return

    # INTERFACE (not part of base class)
    def LoadLog( s ):
        """ Load log into text widget. """
        log = st.log.Read()                  # returns a copy (not a reference)
        s.logLen = len( log )                # used by TextWid.__IdleLog()
        tokenList = [ (eTokenText,log) ]
        fname = "[results: log]"
        contents = ContentsLog( tokenList=tokenList, fname=fname )
        line = log.count( "\n" ) - 1         # find last line to show end of log
        s.LoadContents( contents=contents, line=line )

    # PRIVATE
    def __IdleLog( s ):
        # If log is loaded in a text widget, automatically refresh it.
        if isinstance(s.contents,ContentsLog):
            log    = st.log.Read()    # returns a copy (not a reference)
            logLen = len(log)
            if s.logLen < logLen:
                
                # Append new log lines directly to Tk text widget.
                text = log[s.logLen:]     # new log lines
                s.widget.insert( TK.END, text, (st.gui.TKTAG_DEFAULT) )
                s.logLen = logLen         # update log length
                s.widget.see( TK.END )    # scroll down
                
        # Again.
        st.gui.ScheduleTimerFunc( s.milsecIdleLog, s.__IdleLog )
