# $Id: Startup.py,v 1.43 2006-03-26 03:03:32 devel Exp $
# pypersrc
# Author:   Jim Brooks  http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

#-------------------------------------------------------------------------------
# Startup.
#-------------------------------------------------------------------------------

# This appears at the top of the catenated Python script.
def Startup( args ):
    """ Startup function. """
    try:
        # Convention for arg processing is to assign None to elements
        # in argList after processing the corresponding args.
        argList = list(args)[1:] # skip argv[0]
        
        global st
        st = State( argList )

        if not ProcessArgs( argList ):
            return

        st.log.SetSplashLine( "pypersrc log (" + st.version + ") \n" )

        # Create the major objects of pypersrc.
        st.gather  = Gather(  argList )
        st.tags    = Tags(    argList )
        st.lex     = Lex(     argList )
        st.parser  = Parse(   argList )
        st.command = Command( argList )
        st.gui     = Gui(     argList )
        st.threads = Threads( argList )

        # Gather src files, tag their symbols.
        st.threads.Spawn( ThreadGatherTag, argList )
        
        # Enter GUI event loop.
        st.gui.Enable()
        st.command.Do( Command.INTERNAL_COMMAND_SPLASH )
        st.gui.statusbar.Print( st.GetSplashText() )
        st.gui.EventLoop()

        # Return to C++ code (main.cc).
        return
    
    except:
        PrintException()

def ProcessArgs( argList ):
    """ The first pass of cmd-line arg processing. """
    """ Returns False if program should return/exit. """
    try:
        i = 0
        while i < len(argList):
            arg = argList[i]
            if arg is None:
                i += 1
            elif arg == "-setversion":
                # Internal arg passed by main.cc.
                st.version = argList[i+1]
                argList[i]   = None
                argList[i+1] = None
                i += 2
            elif arg == "-l":
                # Programming language arg.
                # Design note: -l <lang> must be followed by exactly one sub-arg.
                # Allowing multiple sub-args to follow would be convenient,
                # but that would open the door to misinterpreting file/dir args.
                # Eg "pypersrc -l c++ ." means to browse the CWD ('.' isn't a language arg).
                argList[i] = None
                i += 1
                st.langList = []
                if i >= len(argList):
                    PrintWarning( "-l missing sub-arg" )
                elif not argList[i] in progLangList:
                    PrintWarning( "'" + argList[i] + "' is an unsupported programming language" )
                else:
                    st.langList.append( argList[i] )
                    argList[i] = None
                    i += 1
            elif arg in [ "-h", "-help", "--help" ]:
                print "pypersrc by Jim Brooks  http://www.jimbrooks.org/hypersrc/"
                print "pypersrc is a source code browser written in Python."
                print "License: GNU GPL"
                print "Command-line arguments:"
                print "file(s)     source files to browse"
                print "dir(s)      pypersrc will recurse into directories"
                print "-x glob     exclude files that match the fileglob pattern"
                print "-l lang     programming language source files to browse, any of:"
                print "            c c++ lua java python"
                print "            example: -l c++ -l java  (default: c c++ lua python)"
                print "-geom       window size"
                print "            example: -geom 800x600"
                print "-ed-gui     GUI editor to use (default: emacs)"
                print "            example: -ed-gui xemacs"
                print "-ed-xterm   console editor to use (default: vi/vim)"
                print "            example: -ed-xterm joe"
                print "-hilite-lo[hi] less[more] syntax hiliting (affects speed of loading files)"
                return False
            elif arg in [ "-ed-gui", "-editor-gui" ]:
                st.editorGui = argList[i+1]
                argList[i]   = None
                argList[i+1] = None
                i += 2
            elif arg in [ "-ed-xterm", "-editor-xterm", "-ed-console" ]:
                st.editorXterm = argList[i+1]
                argList[i]   = None
                argList[i+1] = None
                i += 2
            else:
                i += 1
    except:
        PrintException()
    
    return True
