# $Id: State.py,v 1.32 2006-03-26 03:03:32 devel Exp $
# pypersrc
# Author:   Jim Brooks  http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

#-------------------------------------------------------------------------------
# Class State.
#-------------------------------------------------------------------------------

class State(object):

    def __init__( s, argList ):
        """ Global state. """

        s.log             = Log()    # first of all
        s.mainThreadId    = thread.get_ident()

        s.ProcessArgsPre( argList )
        
        # ---- PERSISTENT STATE (TODO) ------------------------------
        s.visited         = {}    # visited hyperlinks
        # ---- PERSISTENT STATE (TODO) ------------------------------
        
        # s.langList is the default list of programming language files to gather.
        # The -l arg can append to it.
        s.version         = ""    # assigned by C++ module passing -setversion internal arg
        s.exiting         = False   # set when GUI destroy event (quit/program exit) is dequeued
        s.error           = False   # set by PrintError()
        s.warning         = False   # set by PrintWarning()
        s.refreshCnt      = 0       # to recompute stale results
        s.langList        = [ "c", "c++", "lua", "python" ]  # -- default programming languages --
        s.verbose         = False
        s.ignoreDirList   = [ "CVS", "RCS", "bak", "out" ]
        s.tabWidth        = 4
        s.escKey          = False
        # Two kinds of editors are supported: console and GUI (eg vi and xemacs).
        s.editorGui       = "emacs"
        s.editorXterm     = "vi"
        # Prefer vim over vi.
        for pname in [ "/usr/bin/vim", "/usr/local/bin/vim" ]:
            if os.access( pname, os.F_OK ):
                s.editorXterm = "vim"
                break
        if IfCygwin():
            # Must be an absolute pathname (limited by Python/Windows spawn()).
            s.editorGui   = "c:\\program files\\windows nt\\accessories\\wordpad.exe"
            if not os.path.isfile( s.editorGui ):
                s.log.Write( "Startup: can't find editor '" + s.editorGui + "'" )
                s.editorGui  = None
            s.editorXterm = None
        elif IfDarwin():
            s.editorGui   = "xemacs"
            s.editorXterm = "vi"
        
        s.ProcessArgsPost( argList )

    # INTERFACE
    def ProcessArgsPre( s, argList ):
        """ Assigns None to elements in argList that were processed. """
        return True
    
    # INTERFACE
    def ProcessArgsPost( s, argList ):
        """ Assigns None to elements in argList that were processed. """
        i = 0
        while i < len(argList):
            arg = argList[i]
            if arg is None:
                i += 1
            elif arg == "-v":
                s.verbose = True
                s.log.SetLogLevel( Log.VERBOSE )
                argList[i] = None
                i += 1
            else:
                i += 1
        return True

    # INTERFACE
    def IfMainThread( s ):
        """ Return True if this is the main thread. """
        return s.mainThreadId == thread.get_ident()

    # INTERFACE
    def GetSplashText( s ):
        # copyright
        return "pypersrc " + s.version + "  (c) Jim E. Brooks 2005-2009  http://www.jimbrooks.org/hypersrc/  GNU GPL no warranty"
