# $Id: testDequeChar.py,v 1.4 2006-03-14 13:45:55 devel Exp $
# Test Python DequeChar and underlying C++ DequeChar.
# Author:   Jim E. Brooks http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

# This test is meant to be run by the C++ executable (not directly)
# when it is passed the cmd-line arg "-test".
# For example: pypersrc -test testDequeChar   # [.py] is implied
# See src/cc/main.cc.

print "Loading DequeChar test..."
import sys, os, traceback
from string import *
sys.path.append( "src/py" )
sys.path.append( "../src/py" )
from lib import *
try:
    from DequeChar import *
except:
    print "ERROR: Run this using the C++ executable pypersrc with the args -test testDequeChar"

def Startup( *args ):
    success = False
    try:
        print "Running test..."

        deq = DequeChar( "abcdef" )

        ###########################################################
        print "Testing primary cases... "
        
        deq.Set( "abc" )
        if deq.Size() != 3:
            raise Oops
        text = deq.Get()
        if text != "abc":
            raise Oops

        # Set with large string.
        sz = 32768
        deq.Set( "a" * sz )
        if deq.Size() != sz:
            raise Oops
        if deq.Get() != "a" * sz:
            raise Oops

        # Clear deque.
        deq.Set( "" )
        if deq.Size() != 0:
            raise Oops

        #############################################################
        print "Testing peek,pop at head..."
        
        deq.Set( "abcdef" )
        
        text = deq.PeekHead( 3 )
        print "Testing PeekHead(3) = '" + text + "'"
        if text != "abc" or deq.Size() != 6:
            raise Oops

        text = deq.PopHead( 3 )
        print "Testing PopHead(3) = '" + text + "'"
        if text != "abc" or deq.Size() != 3:
            raise Oops

        text = deq.PopHead( 2 )
        print "Testing PopHead(2) = '" + text + "'"
        if text != "de" or deq.Size() != 1:
            raise Oops

        text = deq.PopHead( 1 )
        print "Testing PopHead(1) = '" + text + "'"
        if text != "f" or deq.Size() != 0:
            raise Oops

        xcpt = 0
        try:
            text = deq.PopHead( 1 )
        except:
            print "PopHead(1) caused exception (expected)"
            xcpt = 1
        if not xcpt:
            raise Oops

        ###################################################
        print "Testing peep,pop at tail..."

        deq.Set( "abcdef" )
        
        text = deq.PeekTail( 3 )
        print "Testing PeekTail(3) = '" + text + "'"
        if text != "def" or deq.Size() != 6:
            raise Oops

        text = deq.PopTail( 3 )
        print "Testing PopTail(3) = '" + text + "'"
        if text != "def" or deq.Size() != 3:
            raise Oops

        ####################################################
        print "Testing push/pop at head..."

        deq.Set( "" )

        text = "1234"
        deq.PushHead( text )
        print "Testing PushHead('" + text + "')"
        if deq.Size() != 4:
            raise Oops

        text = deq.PopHead( 2 )
        print "Testing PopHead(2) = '" + text + "'"
        if text != "12" or deq.Size() != 2:
            raise Oops

        ######################################################
        print "Testing push/pop at tail..."

        deq.Set( "" )
        
        text = "abcd"
        deq.PushTail( text )
        print "Testing PushTail('" + text + "')"
        if deq.Size() != 4:
            raise Oops

        text = deq.PopTail( 2 )
        print "Testing PopTail(2) = '" + text + "'"
        if text != "cd" or deq.Size() != 2:
            raise Oops

        ################################################
        print "Testing __getitem__() and iteration...."

        text = "abcdef12345"
        deq.Set( text )
        
        if deq.Size() != len(text):
            raise Oops

        if deq[0] != text[0]:
            raise Oops

        if deq[1] != text[1]:
            raise Oops

        if deq[-1] != text[-1]:
            raise Oops

        if deq[-2] != text[-2]:
            raise Oops

        xcpt = 0
        try:
            c = deq[-999]
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        i = 0
        for c in text:
            if c != deq[i]:         # iterate with positive indexes
                raise Oops
            i += 1

        i = -1
        while i >= -len(text):      # iterate with negative indexes
            if text[i] != deq[i]:
                raise Oops
            i -= 1

        # By defining deq.__getinit__(), iteration automatically works.
        # Cool.
        i = 0
        for c in deq:
            if c != deq[i]:
                raise Oops
            i += 1

        ################################################
        print "Testing __setitem__()..."

        text = "12345"
        deq.Set( text )

        deq[0] = "X"
        if deq[0] != "X":
            raise Oops
        if deq.Get() != "X2345":
            raise Oops

        deq[4] = "Y"
        if deq[4] != "Y":
            raise Oops
        if deq.Get() != "X234Y":
            raise Oops

        deq[-2] = "Z"
        if deq[-2] != "Z":
            raise Oops
        if deq.Get() != "X23ZY":
            raise Oops

        deq[-5] = "J"
        if deq[-5] != "J":
            raise Oops
        if deq.Get() != "J23ZY":
            raise Oops

        xcpt = 0
        try:
            deq[5] = "X"
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            deq[-6] = "X"
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        ###################################################
        print "Testing Index()..."

        text = "dogcatbird"  # [0]=dog [3]=cat [6]=bird len=10
        deq.Set( text )

        ####################################################
        print "Testing normal Index() cases..."

        idx = deq.Index( "dog" )
        if idx != 0:
            raise Oops

        idx = deq.Index( "cat" )
        if idx != 3:
            raise Oops

        idx = deq.Index( "cat", 1 )
        if idx != 3:
            raise Oops

        idx = deq.Index( "bird" )
        if idx != 6:
            raise Oops

        idx = deq.Index( "dogcatbird" )
        if idx != 0:
            raise Oops

        idx = deq.Index( "dogcatbird", 0 )
        if idx != 0:
            raise Oops

        idx = deq.Index( "a" )
        if idx != 4:
            raise Oops

        idx = deq.Index( "rd", 8 )  # ok
        if idx != 8:
            raise Oops

        #######################################################
        print "Testing Index() cases with duplicate substrings..."

        deq.Set( "aaadogdog" )    # duplicate substrings
        idx = deq.Index( "dog" )
        if idx != 3:
            raise Oops
        deq.Set( text )

        deq.Set( "aaadogdog" )    # duplicate substrings
        idx = deq.Index( "dog", 3 )
        if idx != 3:
            raise Oops
        deq.Set( text )

        deq.Set( "aaadogdog" )    # duplicate substrings
        idx = deq.Index( "dog", 4 )
        if idx != 6:
            raise Oops
        deq.Set( text )

        ###################################################
        print "Testing exceptional Index() cases..."

        xcpt = 0
        try:
            idx = deq.Index( "rdd", 8 )  # sought string is 1 char too much
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.Index( "dogcatbirdd", 0 )  # sought string is 1 char too much
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.Index( "" )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.Index( "bird", 100 )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.Index( "bird", -1 )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        #################################################
        print "Testing other exceptional cases..."

        xcpt = 0
        try:
            idx = deq.PeekHead( 0 )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.PopHead( -1 )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.PopTail( -1 )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            idx = deq.PeekTail( 0 )
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        success = True

    except:
        PrintException()

    print "----------"
    if success:
        print "Test passed."
    else:
        print "Test FAILED!"
    return
