/*****************************************************************************
 * $Id: testFind.cc,v 1.2 2006-02-28 14:24:37 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Test utils_string.cc:Find().
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) JIM E. BROOKS 2006
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#include <iostream>
using namespace std;

#include "utils_string.hh"
using namespace UtilsNS;

bool
TestCase( const char* haystack, const char* needle, long expectedIdx, bool caseSens )
{
	int line = 1; // Find() requires initializing as 1
	const char* match = Find( haystack, needle, caseSens, line );
	if ( expectedIdx == -1 ) // means NULL is expected
	{
		if ( match == NULL ) return true;
	}
	else
	{
		if ( match - haystack == expectedIdx ) return true;
	}
	cerr << "FAILED: haystack='" << haystack << "' needle='" << needle << "' expectedIdx=" << expectedIdx << endl;
	return false;
}

bool
TestFindCI( void )
{
	cerr << "Testing case-insensitive...\n";
	// Remember these should be case-insensitive.
	bool caseSens = false;
	if ( ! TestCase( "abc123", "123", 3, caseSens ) ) return false;
	if ( ! TestCase( "abc123", "123", 3, caseSens ) ) return false;
	if ( ! TestCase( "abc123123", "123", 3, caseSens ) ) return false;
	if ( ! TestCase( "ababaa", "aa", 4, caseSens ) ) return false;
	if ( ! TestCase( "aaaAZ", "AZ", 3, caseSens ) ) return false;
	if ( ! TestCase( "xyzXYZ", "XYZ", 0, caseSens ) ) return false; // should be 0 as this is case-insensitive
	if ( ! TestCase( "xyzxyz", "xyz", 0, caseSens ) ) return false;
	if ( ! TestCase( "XYZxyz", "xyz", 0, caseSens ) ) return false;
	if ( ! TestCase( "ababba", "ba", 1, caseSens ) ) return false;
	if ( ! TestCase( "ABABBA", "ba", 1, caseSens ) ) return false;
	if ( ! TestCase( "ABABBA", "bA", 1, caseSens ) ) return false;
	if ( ! TestCase( "aBABBA", "Ba", 1, caseSens ) ) return false;
	if ( ! TestCase( "abcdef", "xyz", -1, caseSens ) ) return false; // -1 means NULL is expected for no match
	if ( ! TestCase( "", "xyz", -1, caseSens ) ) return false;
	if ( ! TestCase( "", "", -1, caseSens ) ) return false;
	if ( ! TestCase( "xyz", "", -1, caseSens ) ) return false;
	return true;
}

bool
TestFindCS( void )
{
	cerr << "Testing case-sensitive...\n";
	// Remember these should be case-sensitive.
	bool caseSens = true;
	if ( ! TestCase( "abcABC", "abc", 0, caseSens ) ) return false;
	if ( ! TestCase( "abcABCABc", "ABc", 6, caseSens ) ) return false;
	if ( ! TestCase( "abcABC", "ABc", -1, caseSens ) ) return false;
	return true;
}

int main( int argc, char** argv )
{
	if ( TestFindCI() && TestFindCS() )
	{
		cerr << "\nTested passed. \n";
		exit(0);
	}
	else
	{
		cerr << "\nTested FAILED! \n";
		exit(1);
	}
}
