# $Id: testGreedyDict.py,v 1.9 2006-03-14 13:45:55 devel Exp $
# Test Python GreedyDict and underlying C++ GreedyMap.
# Author:   Jim E. Brooks http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2005
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

# This test is meant to be run by the C++ executable (not directly)
# when it is passed the cmd-line arg "-test".
# For example: pypersrc -test testGreedyDict   # [.py] is implied
# See src/cc/main.cc.

print "Loading GreedyDict test..."
import sys, os, traceback
from string import *
sys.path.append( "src/py" )
from lib import *
try:
    from GreedyDict import *
except:
    print "ERROR: Run this using the C++ executable pypersrc with the args -test testGreedyDict"

def Startup( *args ):
    try:
        print "Running test..."
        
        #########################################
        
        print "Testing [a0,b1] ..."
        greedy = GreedyDict( 10 )
        
        greedy["a0"] = 123
        greedy["b1"] = 456
        
        if greedy["a0"] != 123:
            raise Oops
        if greedy["b1"] != 456:
            raise Oops

        greedy["b1"] = 777            # should overwrite old value
        if greedy["b1"] != 777:
            raise Oops

        xcpt = 0
        try:
            x = greedy["a1"]
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        xcpt = 0
        try:
            x = greedy["b0"]
        except:
            xcpt = 1
        if not xcpt:
            raise Oops

        #########################################
        
        print "Testing [cat,rat] ..."

        greedy = GreedyDict( 50 )
        
        greedy["cat"] = "feline"
        greedy["rat"] = "rodent"

        if greedy["cat"] != "feline":
            raise Oops
        if greedy["rat"] != "rodent":
            raise Oops

        #########################################
        
        print "Testing [greedy matching] ..."

        greedy = GreedyDict( 10 )
        
        greedy["cat"]    = "feline"
        greedy["rat"]    = "rodent"
        greedy["ratcat"] = "hybrid"

        if greedy["cat"] != "feline":
            raise Oops
        if greedy["rat"] != "rodent":
            raise Oops
        if greedy["ratcat"] != "hybrid":  # should find longest match (greedy)
            raise Oops                    # it shouldn't stop at "rat"
        if greedy["rat"] != "rodent":
            raise Oops
        if greedy["catxxx"] != "feline":  # should still match
            raise Oops
        if greedy["ratcatZZZZ"] != "hybrid":  # should still match
            raise Oops

        #########################################
        
        print "Testing [Find(), heterogeneous objects] ..."

        greedy = GreedyDict( 10 )

        greedy["aa1"]     = 123
        greedy["bbaa1"]   = "zzz"
        greedy["?"]       = ("q","q")

        if greedy["aa1"] != 123:
            raise Oops
        if greedy["bbaa1"] != "zzz":
            raise Oops
        if greedy["?"] != ("q","q"):
            raise Oops

        #########################################
        
        print "Testing [C++ keywords] ..."

        eTokenKeyword = 1
        eTokenKeywordLoud = 2
        eTokenType = 3
        keywordDict = { "return"          : eTokenKeyword,
                        "if"                : eTokenKeyword,
                        "else"              : eTokenKeyword,
                        "return"            : eTokenKeyword,
                        "for"               : eTokenKeyword,
                        "while"             : eTokenKeyword,
                        "do"                : eTokenKeyword,
                        "continue"          : eTokenKeyword,
                        "break"             : eTokenKeyword,
                        "switch"            : eTokenKeyword,
                        "case"              : eTokenKeyword,
                        "default"           : eTokenKeyword,
                        "sizeof"            : eTokenKeyword,
                        "goto"              : eTokenKeywordLoud,
                        "NULL"              : eTokenKeywordLoud,
                        "asm"               : eTokenKeyword,
                        # C pre-processsor
                        "#define"           : eTokenKeyword,
                        "#ifdef"            : eTokenKeyword,
                        "#ifndef"           : eTokenKeyword,
                        "#if"               : eTokenKeyword,
                        "#else"             : eTokenKeyword,
                        "#elif"             : eTokenKeyword,
                        "#endif"            : eTokenKeyword,
                        "#include"          : eTokenKeyword,
                        "#undef"            : eTokenKeyword,
                        "#error"            : eTokenKeyword,
                        "#pragma"           : eTokenKeyword,
                        "defined"           : eTokenKeyword,
                        # C++
                        "catch"             : eTokenKeyword,
                        "delete"            : eTokenKeyword,
                        "namespace"         : eTokenKeyword,
                        "new"               : eTokenKeyword,
                        "operator"          : eTokenKeyword,
                        "private"           : eTokenKeyword,
                        "public"            : eTokenKeyword,
                        "this"              : eTokenKeyword,
                        "throw"             : eTokenKeyword,
                        "try"               : eTokenKeyword,
                        "typeid"            : eTokenKeyword,
                        "typename"          : eTokenKeyword,
                        "static_cast"       : eTokenKeywordLoud,
                        "const_cast"        : eTokenKeywordLoud,
                        "reinterpret_cast"  : eTokenKeywordLoud,
                        # C types
                        "void"              : eTokenType,
                        "char"              : eTokenType,
                        "short"             : eTokenType,
                        "int"               : eTokenType,
                        "long"              : eTokenType,
                        "float"             : eTokenType,
                        "double"            : eTokenType,
                        "const"             : eTokenType,
                        "volatile"          : eTokenType,
                        "register"          : eTokenType,
                        "static"            : eTokenType,
                        "extern"            : eTokenType,
                        "signed"            : eTokenType,
                        "unsigned"          : eTokenType,
                        "struct"            : eTokenType,
                        "union"             : eTokenType,
                        "typedef"           : eTokenType,
                        "enum"              : eTokenType,
                        # C++ types
                        "bool"              : eTokenType,
                        "class"             : eTokenType,
                        "explicit"          : eTokenType,
                        "export"            : eTokenType,
                        "friend"            : eTokenType,
                        "mutable"           : eTokenType,
                        "protected"         : eTokenType,
                        "template"          : eTokenType,
                        "virtual"           : eTokenType,
                        "inline"            : eTokenType,
                        # C++ STL types
                        "string"            : eTokenType,
                        "vector"            : eTokenType,
                        "deque"             : eTokenType,
                        "set"               : eTokenType,
                        "multiset"          : eTokenType,
                        "map"               : eTokenType,
                        "multimap"          : eTokenType,
                        "iterator"          : eTokenType,
                        "const_iterator"    : eTokenType,
                        "size_type"         : eTokenType,
                        }

        greedyDict = GreedyDict( 40 )
        for (key,val) in keywordDict.items():
            greedyDict[key] = val

        (key,val) = greedyDict.Find( "class" )
        if key != "class":
            raise Oops

        (key,val) = greedyDict.Find( "template" )
        if key != "template":
            raise Oops

        xcpt = 0
        try:
            (key,val) = greedyDict.Find( "mplate<class Val>" )  # a bug caused this to match this as
        except:                                                 # "multimap" (should be fixed now)
            xcpt = 1
        if not xcpt:
            raise Oops

        for (key,val) in keywordDict.items():
            (key2,val2) = greedyDict.Find( key )
            if key != key2:
                raise Oops

        #########################################
        
        print "Testing [garbage collection] ..."

        i = 300
        while i:
            i -= 1
            greedy = GreedyDict( 20 )
            j = 200
            while j:
                j -= 1
                greedy["abc"] = "a" * 200

        #########################################

        print "Test passed."
        return

    except:
        PrintException()
