/*****************************************************************************
 * $Id: testGreedyMap.cc,v 1.2 2006-02-25 06:01:17 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) JIM E. BROOKS 2005
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#include <iostream>
using namespace std;

#include "greedyMap.hh"

static int callbackMatches = 0;
void GreedyStringCallback( const string& key, const string& val );
void GreedyPtrCallback( const string& key, void* const& val );

bool
GreedyMapTest( void )
{
	bool res;

	//---------------------------------------------
 {
	cerr << "Testing GreedyMap<string>... \n";
	GreedyMap<string> greed( 10 );
	string keyOut;
	string valOut;
	string valOld;

    // Ensure this comment in GreedyDict.cc is true:
    // As an example of both greedy and partial matching,
    // assuming { "ab", "abc", "abcd" } are stored in the map,
    // the pattern "abc123" will be matched with the key "abc".
	if ( greed.Insert( string("ab"), string("_ab"), valOld ) )
		return false;
	if ( greed.Insert( string("abc"), string("_abc"), valOld ) )
		return false;
	if ( greed.Insert( string("abcd"), string("_abcd"), valOld ) )
		return false;
	res = greed.Find( string("abc123"), keyOut, valOut );
	if ( ! res || keyOut != "abc" || valOut != "_abc" )
		return false;

    // Various test cases.
	if ( greed.Insert( string("cat"), string("feline"), valOld ) )
		return false;
	res = greed.Find( string("cat"), keyOut, valOut );
	if ( ! res || keyOut != "cat" || valOut != "feline" )
		return false;

	if ( greed.Insert( string("rat"), string("rodent"), valOld ) )	// cat/rat case
		return false;
	res = greed.Find( string("rat"), keyOut, valOut );
	if ( ! res || keyOut != "rat" || valOut != "rodent" )
		return false;

	if ( greed.Insert( string("ratxx"), string("rodent"), valOld ) ) // still should match
		return false;
	res = greed.Find( string("rat"), keyOut, valOut );
	if ( ! res || keyOut != "rat" || valOut != "rodent" )
		return false;

	if ( greed.Insert( string("ratcat"), string("hybrid"), valOld ) )	// test greedy match
		return false;
	res = greed.Find( string("ratcat"), keyOut, valOut );
	if ( ! res || keyOut != "ratcat" || valOut != "hybrid" )
		return false;

	if ( greed.Insert( string("ratcatxx"), string("hybrid"), valOld ) )	// still should match
		return false;
	res = greed.Find( string("ratcat"), keyOut, valOut );
	if ( ! res || keyOut != "ratcat" || valOut != "hybrid" )
		return false;

	res = greed.Find( string("ratcatZZZ"), keyOut, valOut );	// should still match
	if ( ! res || keyOut != "ratcat" || valOut != "hybrid" )
		return false;

	res = greed.Find( string("xrat"), keyOut, valOut );			// shouldn't match
	if ( res )
		return false;

	if ( greed.Insert( string("1234567890134567890"), string("num"), valOld ) )  // causes vector to extend
		return false;
	res = greed.Find( string("1234567890134567890"), keyOut, valOut );
	if ( ! res || keyOut != "1234567890134567890" || valOut != "num" )
		return false;
 }

	//---------------------------------------------
 {
	cerr << "Testing GreedyMap<string> #2 ... \n";
	GreedyMap<string> greed( 10 );
	string keyOut;
	string valOut;
	string valOld;

	if ( greed.Insert( string("a1"), string("123"), valOld ) )
		return false;
	if ( greed.Insert( string("b2"), string("456"), valOld ) )
		return false;

	res = greed.Find( string("a1"), keyOut, valOut );
	if ( ! res || keyOut != "a1" || valOut != "123" )
		return false;

	res = greed.Find( string("b2"), keyOut, valOut );
	if ( ! res || keyOut != "b2" || valOut != "456" )
		return false;

	res = greed.Find( string("a2"), keyOut, valOut );
	if ( res )
		return false;

	res = greed.Find( string("b1"), keyOut, valOut );
	if ( res )
		return false;
 }

	//---------------------------------------------
 {
	cerr << "Testing GreedyMap<string> #3 ... \n";
	GreedyMap<string> greed( 10 );
	string keyOut;
	string valOut;
	string valOld;

	if ( greed.Insert( string("cat"), string("feline"), valOld ) )
		return false;
	if ( ! greed.Insert( string("cat"), string("overwrite"), valOld ) ) // should overwrite
		return false;

	res = greed.Find( string("cat"), keyOut, valOut );
	if ( ! res || keyOut != "cat" || valOut != "overwrite" )
		return false;
 }

	//---------------------------------------------

 {
	cerr << "Testing GreedyMap<int>... \n";
	GreedyMap<int> greed( 10 );
	string keyOut;
	int valOut;
	int valOld;

	if ( greed.Insert( string("Digger Barnes' bank account"), 100, valOld ) )
		return false;
	res = greed.Find( string("Digger Barnes' bank account"), keyOut, valOut );
	if ( ! res || keyOut != "Digger Barnes' bank account" || valOut != 100 )
		return false;

	if ( greed.Insert( string("J. R. Ewing's bank account"), 100000000, valOld ) )
		return false;
	res = greed.Find( string("J. R. Ewing's bank account"), keyOut, valOut );
	if ( ! res || keyOut != "J. R. Ewing's bank account" || valOut != 100000000 )
		return false;
 }

	//---------------------------------------------

 {
	cerr << "Testing GreedyMap<void*>... \n";
	GreedyMap<void*> greed( 100 );
	string keyOut;
	void* valOut;
	void* valOld;

	if ( greed.Insert( string("o1"), (void*)100, valOld ) )
		return false;
	res = greed.Find( string("o1"), keyOut, valOut );
	if ( ! res || keyOut != "o1" || valOut != (void*)100 )
		return false;
 }

	//---------------------------------------------

 {
	cerr << "Testing GreedyMap ForEach() callback... \n";
	GreedyMap<string> greed( 10 );
	string valOld;

	callbackMatches = 0;
	greed.Insert( string("cat"), string("feline"), valOld );
	greed.Insert( string("rat"), string("rodent"), valOld );
	greed.ForEach( GreedyStringCallback );
	if ( callbackMatches != 2 )
		return false;
 }

	//---------------------------------------------

 {
	GreedyMap<string> greed( 10 );

	callbackMatches = 0;
	greed.ForEach( GreedyStringCallback );
	if ( callbackMatches != 0 )
		return false;
 }

	//---------------------------------------------

 {
	GreedyMap<string> greed( 10 );

	string valOld;

	if ( greed.Insert( string("cat"), string("feline"), valOld ) )
		return false;
	if ( ! greed.Insert( string("cat"), string("feline"), valOld ) )  // again, should overwrite
		return false;

	callbackMatches = 0;
	greed.ForEach( GreedyStringCallback );
	if ( callbackMatches != 1 )
		return false;
 }

	//---------------------------------------------

 {
	cerr << "Testing GreedyMap ForEach() callback #2... \n";
	GreedyMap<void*> greed( 10 );
	void* valOld;

	callbackMatches = 0;
	if ( greed.Insert( string("n1"), (void*)123, valOld ) )
		return false;
	if ( greed.Insert( string("n2"), (void*)456, valOld ) )
		return false;
	greed.ForEach( GreedyPtrCallback );
	if ( callbackMatches != 2 )
		return false;
 }

	return true;
}

void
GreedyStringCallback( const string& key, const string& val )
{
	callbackMatches += (key=="cat"&&val=="feline") || (key=="rat"&&val=="rodent");
}

// Second arg is tricky to declare: it's a const pointer, not pointer-to-const.
void
GreedyPtrCallback( const string& key, void* const& val )
{
	callbackMatches += (key=="n1"&&long(val)==123) || (key=="n2"&&long(val)==456);
}

int main( int argc, char** argv )
{
	if ( GreedyMapTest() )
	{
		cerr << "\nTested passed. \n";
		exit(0);
	}
	else
	{
		cerr << "\nTested FAILED! \n";
		exit(1);
	}
}
