# $Id: testLib.py,v 1.2 2006-02-26 22:40:01 devel Exp $
# Test lib.py.
# Author:   Jim E. Brooks http://www.jimbrooks.org
# LEGAL:    COPYRIGHT (C) JIM E. BROOKS 2006
#           THIS SOURCE CODE IS RELEASED UNDER THE TERMS
#           OF THE GNU GENERAL PUBLIC LICENSE (GPL).
# ==============================================================================

# This test can be run directly by Python (it doesn't use C++ modules):
# python testLib.py

import sys, os, traceback
from string import *
sys.path.append( "src/py" )
sys.path.append( "../src/py" )
from lib import *

class OopsTestFail:
    pass

# ------------------------------------------------------------------------------
# Test SplitWords()
# ------------------------------------------------------------------------------

def Test_SplitWords( wordStr, wordListExpected, keepOuterQuotes=True ):
    wordList = SplitWords( wordStr, keepOuterQuotes )
    #print wordList
    if wordListExpected != None and wordListExpected != wordList:
        raise OopsTestFail

print "Testing SplitWords()..."
Test_SplitWords( "abc", [ "abc" ] )
Test_SplitWords( "abc ", [ "abc" ] )
Test_SplitWords( " abc", [ "abc" ] )
Test_SplitWords( " abc", [ "abc" ] )
Test_SplitWords( "a b c", [ "a", "b", "c" ] )
Test_SplitWords( "  X   Y   xyz  ", [ "X", "Y", "xyz" ] )
Test_SplitWords( "'a'", [ "'a'" ] )
Test_SplitWords( "'a' bc", [ "'a'", "bc" ] )
Test_SplitWords( "'a b c' de ", [ "'a b c'", "de" ] )
Test_SplitWords( " \t ' x ' \t ", [ "' x '" ] )
Test_SplitWords( 'f "abc == " "my project/*"', [ "f", "\"abc == \"", "\"my project/*\"" ] )
# Special case: pair of quotes:
Test_SplitWords( 'f ""hilite""', [ 'f', '""hilite""' ] )
Test_SplitWords( "f ''hilite''", [ "f", "''hilite''" ] )
Test_SplitWords( "f \'\"hilite\"\'", [ "f", "\'\"hilite\"\'" ] )
Test_SplitWords( 'f ""abc == "" "my project/*"', [ 'f', "\"\"abc == \"\"", "\"my project/*\"" ] )
Test_SplitWords( 'f \"\'abc == \'\"  "my project/*"', [ 'f', "\"\'abc == \'\"", "\"my project/*\"" ] )
Test_SplitWords( "f \'\"abc == \"\'  'my project/*'", [ 'f', "\'\"abc == \"\'", "\'my project/*\'" ] )
Test_SplitWords( "\"\"Hello World\"\"", [ "\"\"Hello World\"\"" ] )
Test_SplitWords( "'\"\"Hello World\"\"'", [ "'\"\"Hello World\"\"'" ] )
# With keepOuterQuotes=False:
Test_SplitWords( "abc", [ "abc" ], keepOuterQuotes=False )
Test_SplitWords( "'abc'", [ "abc" ], keepOuterQuotes=False )
Test_SplitWords( 'f ""hilite""', [ 'f', '"hilite"' ], keepOuterQuotes=False )
Test_SplitWords( "f ''hilite''", [ "f", "'hilite'" ], keepOuterQuotes=False )
Test_SplitWords( "f \'\"abc == \"\'  'my project/*'", [ 'f', "\"abc == \"", "my project/*" ], keepOuterQuotes=False )
# Corner cases:
Test_SplitWords( "", [] )
Test_SplitWords( " ", [] )
Test_SplitWords( "\t", [] )
# Non-sensical cases but SplitWords() shouldn't raise an exception.
Test_SplitWords( '""', None )
Test_SplitWords( "\"\'", None )

# ------------------------------------------------------------------------------
# End of test.
# ------------------------------------------------------------------------------

print "Test passed."
sys.exit( 0 )
